/***********************************************************************************
 *  CLASE CASILLA
 *  Clase que representa las casillas de un tablero de street master's. Es la 
 *  clase base de la que derivarn todos los tipos de casilla en los que se centrar
 *  la accin del juego.
 *  Una casilla base debe conocer los datos comunes a todas (nombre y nmero), as 
 *  como los jugadores que tiene sobre ella (ahorrando tiempo computacional en su
 *  bsqueda).
 *
 *  Requiere:
 *     - clase jugador ( jerarquia )
 *     - clase algorithm (STL)
 *     - clase vector (STL)
 *     - clase string (STL)
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __CASILLA__
#define __CASILLA__

#include "cadenas.h"	//Libreria creada para el tratamiento de cadenas
#include "jugador.h"

#include <iostream>
#include <cctype>
#include <algorithm>
#include <vector>
#include <string>
using namespace std;

class jugador;		//Forward declaration

class casilla {

 private:
  //Atributos
  int _numero;		            //nmero de casilla
  string _nombre;                   //nombre de la casilla
  string _nombreCorto;              //nombre corto de la casilla

 protected:  
  vector <jugador*> estaJugador;    //Jugadores que se encuentran en la casilla.

 public:

  // Constructores
  casilla ();  
  casilla ( int numero, string nombre, string nombreCorto );
  
  // Copy constructor
  casilla (const casilla& right);

  // Sobrecarga de operadores
  // Operador de asignacin
  casilla& operator= (const casilla& right);

  //operador de insercin
  friend ostream& operator<< (ostream& os, casilla* c);

  //operador de extraccin
  friend istream& operator>> (istream& is, const casilla& c);

  //Opreador distinto
  bool operator!= ( const casilla& c);
  //Operador igual
  bool operator== ( const casilla& c);

  // Destructor
  ~casilla ();

  // Mtodos GET
  int get_numero () const;
  string get_nombre () const;
  string get_nombreCorto () const;

  // Mtodos SET
  void set_numero (int value);
  void set_nombre ( string nombre );
  void set_nombreCorto ( string nombreCorto );

  // MTODOS

  // Mtodos virtuales (polimorfismos)
  /**
   * Mtodo que devuelve en una cadena de caracteres toda la informacin necesaria, con un
   * formato preparado para mostrarse por pantalle,  de una casilla genrica, que 
   * incluye:
   *    -Nombre
   *    -Nmero de casilla
   * En las clases derivadas ser necesario detallar con cuidado esta informacin, ya que
   * existe informacin muy importante..
   */
  virtual string mostrar ();  

  /**
   * Mtodo que efectua la accin propia de cada casilla. Es un mtodo virtual porque cada tipo de
   * casilla tiene una accin propia que debe ejecutar.
   * En el caso de la casilla genrica no se lleva a cabo ninguna accin.
   */
  virtual int accion ( jugador *player );

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  virtual string dibujarCasillaHorizontal ( int fila, int anchoCasilla );

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres.
   */
  virtual string dibujarCasillaVertical ( int fila, int anchoCasilla );
  
  /**
   * Mtodo que devuelve una cadena de caracteres que representa como es la casilla en su 
   * imgen en el tablero, indicando aquellos datos de inters para un jugador que caiga 
   * sobre ella:
   *  -Numero
   *  -Nombre
   */
  virtual string dibujarCasillaEnTablero ();

  /**
   * Devuelve la informacin de la casilla en modo texto para mostrar por pantalla la informacin
   * en forma de tabla. 
   */
  virtual string leyendaCasilla ( int num, int nombreCorto, int tipo, int nombre, int estado );
  
  // Otros mtodos

  /**
   * Mtodo bsico en los movimientos de los jugadores. Coloca un jugador en la casilla actual,
   * insertndolo al final del vector de jugadores.
   */
  void situarJugadorEnCasilla ( jugador* player );

  /**
   * Caso contrario al anterior. Eliminar el jugador de la casilla actual. Buscar en el vector y lo
   * elminacin.
   * Consideracin: No se sabe en que posicin estar el jugador, pero habitualmente estar en la primera
   * ya que normalmente se mueve al jugador que lanza los dados, y ese ha debido ser el primero en llegar
   * all, pero no se puede asegurar, por lo que se realizar una bsqueda secuencial.
   */
  void eliminarJugador ( jugador* player );
};

//Mtodos GET
inline int casilla::get_numero () const
{
  return _numero;
}

inline string casilla::get_nombre () const
{
  return _nombre;
}

inline string casilla::get_nombreCorto () const
{
  return _nombreCorto;
}

//Mtodos SET
inline void casilla::set_numero (int value)
{
  _numero = value;
}

inline void casilla::set_nombre ( string  nombre )
{
  _nombre = nombre;
}

inline void casilla::set_nombreCorto ( string  nombreCorto )
{
  _nombreCorto = nombreCorto;
}

#endif
