/***********************************************************************************
 *  CLASE JUGADOR
 *  Clase que representa un jugador del juego Monopoly. Esta clase es una clase derivada
 *  de la clase propietario y por ello puede poseer propiedades.
 *  A su vez, es la clase base de una nueva clase derivada: jugador_reglas, que es la
 *  clase que contiene las reglas de los agentes de IA.
 *
 *  Requiere:
 *     - clase propietario (herencia)
 *     - clase casilla
 *     - clase dado
 *     - clase tarjeta
 *     - clase negociacion
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __JUGADOR__
#define __JUGADOR__

#include "propietario.h"
#include "casilla.h"
#include "dado.h"

#include <vector>
#include <set>
#include <list>
#include <string>
using namespace std;

class tarjeta;          //forward declaration
class negociacion;      //forward declaration
class casilla;		//forward declaration
class calle;            //forward declaration


class jugador : public propietario {

 private:

  //Atributos
  string _nombre;               //Nombre del jugador
  string _color;                //color de la ficha
  long _dinero_total;           //dinero disponible
  long _capital;                //capital total = dinero_total + inversiones
  int _turnosEncarcelado;       //Turnos en la crcel
  int _tiradasDobles;           //Tiradas dobles consecutivas actualmente
  long _estado;                 //Indica si es moroso / acreedor
  int _multiPago;               //Sus pagos se multiplicarn por esta cantidad

  //Asociaciones   
  casilla *_estaEn;             //Casilla en la que est
  dado *lanza;                  //Dado propio 
  list <tarjeta*> _tieneTarjeta;//Puede tener una o varias tarjetas de suerte


  //LOGS: Vectores de log para sacar las estadsticas del juego
  vector <long> _logDinero;
  vector <int> _logPropiedades;
  vector <int> _logEdificios;
  vector <int> _logConstruibles;
  vector <long> _logCapital;
  
 public:

  /**
   * Constructor por defecto
   */   
  jugador ();
  
  /**
   * Constructor explícito
   */
  jugador ( string nombre, string color, long dinero_total, long capital, int turnosEncarcelados, 
	    int tiradasDobles, long estado, int multiPago );
  
  /**
   * Copy constructor
   */
  jugador (const jugador& right);

  /**
   * Destructor
   */
  ~jugador ();

  /**
   * SOBRECARGA DE OPERADORES
   */

  /**
   * Operador de asignación: Asigno a un jugador los datos de otro
   */
  jugador& operator= (const jugador& right);

  /**
   * Operador de desigualdad. Compara dos jugadores y determina si se trata del
   * mismo jugador. Como los jugadores cuentan con un color de ficha únivoco, se
   * comparará por este atributo.
   */
  bool jugador::operator!= ( const jugador& player );

  friend ostream& operator<< ( ostream& os, jugador *j );


  
  // Mtodos GET 
   
  string get_nombre () const;
  string get_color () const;
  long get_dinero_total () const;
  long get_capital () const;
  int get_turnosEncarcelado () const;
  int get_tiradasDobles () const;
  long get_estado () const;
  casilla* get_estaEn () const;  
  int get_numPropiedades () const;
  int get_multiPago () const;
  list <tarjeta*> get_tieneTarjeta () const;
 
  int obtenerCasilla() const;

  // Mtodos SET
   
  void set_nombre ( string nombre );
  void set_color ( string color );
  void set_tiradasDobles ( int tiradasDobles );
  void set_estado ( long estado );
  void set_estaEn ( casilla *nuevaCasilla );
  void set_turnosEncarcelado ( int turnosEncarcelado );  
  void set_dinero_total ( long dinero_total );
  void set_capital ( long capital );
  void set_multiPago ( int multiPago );
  void incluir_tieneTarjeta ( tarjeta *tjt );
  void set_tieneTarjeta ( list <tarjeta*> tieneTarjeta );


  // Otros mtodos
  
  /**
   * Mtodo que lanza el dado y devuelve el valor de cada uno de
   * los dados lanzados y un booleano que indica si la tirada
   * que se a producido es doble.
   */
  int tirarDado ( bool &tiradaDoble, int &dado1, int &dado2 );

  /**
   * Lanza el dado y devuelve el valor total de la tirada.
   * En esta implementacin slo importa el resultado, por lo que
   * no se devuelve ni se muestra el valor intermedio de cada uno
   * de los dados, ni si se trata de una tirada doble.	
   */
  int tirarDado ();

  /**
   * Devuelve la ultima tirada realizada por el jugador
   */
  int ultimaTirada ();
  
  /**
   * El jugador invierte una cantidad de donero, por lo que se resta la cantidad
   * del dinero total, pero se mantiene en el capital.
   */
  long invertir ( long cantidad );  

  /**
   * Mtodo mediante el cual el jugador recupera una cantidad invertida. Del capital se
   * debe restar lo que costo cuando se invirti y sumar el dinero que entra en caja.
   * Se debe sumar la cantidad que recupera al dinero total.
   */
  long recuperarInversion ( long recupera, long costoAntes );

  /**
   * Mtodo por el cual un jugador paga una cantidad. Si la cantidad es 
   * positiva, el jugador pagar dinero, en caso de ser negativa, el jugador estar
   * cobrando dinero. Al pagar, se debe restar la cantidad del dinero total y del
   * capital. Si no tiene dinero suficiente, se indica que es moroso.
   */
  virtual long pagar ( long cantidad );   
  
  /**
   * Elimina la referencia de la casilla en la que est hacia el propio jugador
   */
  void desvincularDeCasilla();
  
  /**
   * Mtodo que comprueba si el jugador est encarcelado
   */
  bool estaEncarcelado ();

   /**
    * Mtodo que obtiene informacin de todas las propieadades del jugador
    */
  string get_infoPropiedades();

  /**
   * Lista todas las propiedades hipotecables indicando nmero, nombre de propiedad y cantidad de
   * dinero que se obtiene al hipotecarla.
   */
  string listarPropiedadesHipotecables ();
  
  /**
   * Lista todas las propiedades hipotecadas indicando nmero, nombre de propiedad y cantidad de
   * dinero que cuesta deshipotecarla.
   */
  string listarPropiedadesHipotecadas ();

  /**
   * Lista todas las propiedades edificables indicando nmero, nombre de propiedad, precio de un
   * nuevo edificio y los edificios disponibles actualmente.
   */
  string listarCallesConstruibles ();

  /**
   * Lista todas las propiedades edificadas indicando nmero, nombre de propiedad, precio de un
   * edificio y los edificios construidos actualmente.
   */
  string listarCallesConstruidas ();

  /**
   * Obtiene el total de casas que ha construido un jugador sobre sus calles.
   */
  int get_casasConstruidas ();

  /**
   * Obtiene el total de hoteles que ha construido un jugador sobre sus calles.
   */
  int get_hotelesConstruidos ();

  /**
   * Mtodo que devuelve el nmero de tarjetas de suerte / caja que posee un jugador.
   * Si no tiene devuelve 0.
   */
  int tieneTarjetasCarcel ();

  /**
   * Mtodo que devuelve el mazo al que pertenece la primera tarjeta de la lista de tarjetas del jugador
   */
  string mazoTarjetaSuerte ();

  /**
   * El jugador devuelve una tarjeta, es decir, elimina la primera tarjeta de su lista. Antes de eliminarla
   * la devuelve
   */
  tarjeta *devuelveTarjeta ();

  /**
   * El jugador actual inicia un proceso de negociacin con otro jugador indicado como argumento. Para ello
   * invocar a metodos de la clase negociacin
   */
  void negociar ( jugador *receptor );

  /**
   * Devuelve la primera tarjeta de la lista. No la elimina de ella.
   */
  tarjeta *get_tarjeta ();
  
  /**
   * Devuelve la tarjeta de la lista que est en el orden 'orden'. No la elimina de ella.
   */
  tarjeta *get_tarjeta ( int orden ); 

  /**
   * Mtodo que aade una lista de tarjetas a la lista de tarjetas ya existente. Es decir, une dos
   * listas de tarjetas en las posesiones del jugador.
   */
  void anyadirListaTarjetas ( list <tarjeta*> listaTarjetas );

  /**
   * Elimina una lista de tarjetas del jugador.
   */
  void eliminarListaTarjetas ( list <tarjeta*> listaTarjetas );
  
  /**
   * Elimina la primera tarjeta del jugador.
   */
  void eliminarTarjeta ();

  /**
   * Mtodo que elimina todos los edificios de las calles y paga la cantidad estipulada por la
   * venta de todos ellos.
   */
  long expropiarEdificios ();

  /**
   * Mtodo que devuelve una cadena de caracteres sobre el jugador lista para ser guardada en un 
   * fichero de partida guardada. La informacin incluye nombre, color, dinero, capital, turnos 
   * encarcelado, tiradas dobles, estado y posicin actual.
   */
  virtual string guardarDatosPersonales ();

  /**
   * Mtodo que devuelve todos los datos de un jugador para guardarlo en un fichero.
   * Incluye los datos personales del jugador, los datos de sus propiedades y las tarjetas
   * que posee.
   */
  string guardar ();

  /**
   * Mtodo que aade una lista de varias propiedades a las que el jugador ya posee.
   * Se actualizar el capital del jugador.
   */
  void anyadirListaPropiedades ( list <propiedad*> listaPropiedades );

  /**
   * Mtodo polimrfico de la clase propietario. En este caso, se cobra por cada propiedad.
   * Elimina de las posesiones del jugador todas las propiedades que se incluyen en la lista
   * de propiedades pasadas como argumento del mtodo.
   */
  void eliminarListaPropiedades ( list <propiedad*> listaPropiedades );

  // Mtodos virtuales: Se redefinen en los jugadores automticos (jugador_reglas)
  
  /**
   * Mtodo que hace que el jugador puje. Se devolver la cantidad que puja, 0 si pasa o
   * -1 si decide salir de la puja.
   */
  virtual long pujar( propiedad *p, long maxActual, jugador *ganador );

  /**
   * El jugador deber responder si quiere comprar una propiedad o no. La propiedad se pasa como
   * argumento. El jugador lo indicara mediante una resupuesta (s,n).
   */
  virtual bool decidirComprar ( propiedad *p );

  /**
   * El jugador deber responder si quiere pagar para salir de la carcel o no. El jugador lo indicara 
   * mediante una resupuesta (s,n).
   * Es necesario conocer la multa y se pasa el argumento PropiedadesLibres para que el jugador basado en
   * reglas pueda decidir.
   */
  virtual bool decidirSalirPagando ( int tpcPropiedadesLibres, long multa );

  /**
   * El jugador deber responder si quiere salir de la carcel usando una tarjeta. El jugador lo indicara 
   * mediante una resupuesta (s,n).
   * El argumento PropiedadesLibres para que el jugador basado en reglas pueda decidir.
   */
  virtual bool decidirSalirTarjeta ( int tpcPropiedadesLibres );

  /**
   * Cuando un jugador recoja una tarjeta que tenga ms de una opcin, deber decidir cual de ellas
   * quiere ejecutar. Para ello se leera del teclado la opcin que desea ejecutar.
   */
  virtual int decidirAccionTarjeta ( int numAcciones, tarjeta *tjt );

  /**
   * El jugador deber decidir si quiere pagar el tanto por ciento de su capital o la cantidad fija
   * indicada. Esta decisin se toma al caer en un impuesto especial.
   */
  virtual char decidirImpuestoEspecial ( long cantidad, int tpc );

  /**
   * Mtodo que devuelve el nmero de estaciones que posee el jugador.
   */
  int poseeEstacion ();

  /**
   * Mtodo que devuelve el nmero de servicios que posee el jugador.
   */
  int poseeServicio ();

  /**
   * Mtodo que devuelve el nmero de calles edificables que posee el jugador.
   */
  int poseeConstruibles ();


  // Mtodos para el control de logs (estadsticas)
  
  /**
   * Aade el valor actual de cada atributo al vector de logs
   */
  void actualizarLogs ();

  /**
   * Visualizar el log completo de dinero
   */
  void verLogDinero ();
  
  /**
   * Visualizar el log completo de capital
   */
  void verLogCapital ();

   /**
    * Visualizar el log completo de numero de propiedades
    */
  void verLogPropiedades ();

  /**
   * Visualizar el log completo de numero de edificios
   */
  void verLogEdificios ();

  /**
   * Visualizar el log completo de propiedades construibles
   */
  void verLogConstruibles ();
  
  /**
   * Mtodo que guarda todos los logs en los ficheros de registro.
   */
  void guardarLogs ();
  
  /**
   * Finaliza el progreso de todos los logs con valor 0.
   */
  void cerrarLogs ();

  // Mtodos necesarios para el jugador basado en reglas
  
  /**
   * Calcula la suma de la utilidad esperada de los alquileres de una lista de propiedades.
   */
  long utilidadEsperadaAlquiler ( vector <propiedad*> propiedades, int n );
  
  /**
   * Suma las hipotecas de todas las propiedades del vector de propiedades
   */
  long sumaHipoteca ( vector <propiedad*> propiedades );

  /** 
   * Imprime una lista de propiedades indicando adems su perdida estimada y el dinero que
   * se consigue con su hipoteca (venta).
   */
  void imprimir ( vector <propiedad*> candidato );
  
  /**
   * Mtodo que une dos vectores de propiedades
   */
  void unir ( vector <propiedad*> &v1, vector <propiedad*> v2 );
  
  /**
   * Algoritmo recursivo que busca la mejor alternativa de propiedades del jugador a hipotecar. Se debe conseguir
   * una cantidad de dinero superior a la variable cantidad. Las propiedades se introducen en el vector L. 
   * Index es la posicin a partir de la cual se comienza a buscar.
   */
  vector <propiedad*> alternativasHipotecaImprimir ( long cantidad, vector <propiedad*> L, int index, bool imprime );

  /**
   * Mtodo que busca las propiedades hipotecables y buscar la mejor alternativa para superar la cantidad 
   * minimizando las prdidas, usando el mtodo alternativasHipotecarImprimir.
   */
  void calculoUtilidadEsperadaHipotecarImprimir ( long cantidad );

  /**
   * Mtodo que une dos vectores de calles sobre el primero de ellos.
   */
  void unirCalles ( vector <calle*> &v1, vector <calle*> v2 );
  //void unir ( vector <calle*> &v1, vector <calle*> v2 );

  /**
   * Mtodo que calcula la prdida que se estima que se ocasionar si se vende una serie de edificios
   * representada mediante un vector de calles.
   */
  long perdidaEsperadaVenderEdificio ( calle *c, vector <calle*> L, int pos );

  /**
   * Suma el beneficio obtenido por la venta de una serie de edificios identificados por un vector de calles
   */
  long sumaVenta (  vector <calle*> candidato );

  /**
   * Calcula la utilidad esperada que se da al vender una serie de edificios representada por un vector de calles
   */
  long utilidadEsperadaVender ( vector <calle*> L, vector <calle*> candidato );
  
  /**
   * Algoritmo recursivo que busca la mejor alternativa de para vender edificios de un jugador. Se debe conseguir
   * una cantidad de dinero superior a la variable cantidad. Las calles de donde se venden se introducen en el vector L. 
   * Index es la posicin a partir de la cual se comienza a buscar.
   */
  vector <calle*> alternativasVenderImprimir ( long cantidad, vector <calle*> L, int index, bool imprime );
 
  /**
   * Imprime una lista de calles incluyendo su perdida estimada y el dinero que se consigue
   */
  void imprimirCalles ( vector <calle*> original, vector <calle*> candidato );

  /**
   * Mtodo que cuenta los edificios que pertenecen a una calle dentro de una lista de calles
   */
  int contarEdificios (  vector <calle*> candidato, calle* c );  

  /**
   * Se debe comprobar que se cumplan las normas del monopoly:
   *  1- Construcciones escalonadas
   *  2- Casas / hoteles disponibles en la banca
   */
  bool controlFactibilidadVenta ( vector <calle*> candidato );

  /**
   * Mtodo que busca los edificios vendibles y buscar la mejor alternativa para superar la cantidad 
   * minimizando las prdidas, usando el mtodo alternativasVenderImprimir.
   * Una lista de edificios se representa como una lista de calles, cada aparicin de una calle, representa
   * un edificio.
   */
  void calculoUtilidadEsperadaVenderImprimir ( long cantidad );
 
  /**
   * Suma todos los valores de deshipoteca de un vector de propiedades
   */
  long sumaDeshipoteca ( vector <propiedad*> propiedades );

  /**
   * Algoritmo recursivo que busca la mejor alternativa de deshipotecas de propiedades. La cantidad mxima que se 
   * puede invertir es la cantidad que se marca en la variable 'cantidad'. La lista contiene todas las propiedades
   * hipotecadas ordenadas por precio de deshipoteca.
   */
  vector <propiedad*> alternativasDeshipotecarImprimir ( long cantidad, vector <propiedad*> L, int index, bool imprime );
  
  /**
   * Mtodo que busca las propiedades hipotecadas y buscar la mejor combinacin para deshipotecarlas sin
   * superar la cantidad indicada en el atributo cantidad. Utilizar el mtodo alternativasDeshipotecarImprimir.
   */
  void calculoUtilidadEsperadaDeshipotecarImprimir ( long cantidad );
  
  /**
   * Imprime una lista de calles incluyendo su beneficio estimado al construir y la inversin que se debe realizar
   */
  void imprimirCallesConstruir ( vector <calle*> original, vector <calle*> candidato  );

  /**
   * Suma el precio de construir los edificios indicados en la variable candidato.
   */
  long sumaConstruccion (  vector <calle*> candidato );

  /**
   * Mtodo que calcula el beneficio esperado al construir un edificio en la calle c, teniendo
   * una lista de edificios construibles
   */
  long beneficioEsperadoConstruirEdificio ( calle* c, vector <calle*> L, int pos );
  
  /**
   * Mtodo que calcula la utilidad esperada de construir una serie de edificios indicados en la lista candidato.
   */
  long utilidadEsperadaConstruir ( vector <calle*> L, vector <calle*> candidato );

  /**
   * Comprueba que una alternativa a construir cumple con las normas de Monopoly en el mbito de la edificacin
   */
  bool controlFactibilidadConstruir ( vector <calle*> candidato );

  /**
   * Mtodo que busca todas las alternativas para construir de una lista de calles y devuelve la mejor alternativa.
   * Un edificio se representa en la lista por la calle en la que se construir.
   */
  vector <calle*> alternativasConstruirImprimir ( long cantidad, vector <calle*> L, int index, bool imprime );

  /**
   * Mtodo que usando el metodo alternativasConstruirImprimir busca la mejor alternativa para construir sin 
   * sobrepasar la cantidad indicada. Iterativamente buscar la mejor opcin indicando como inicio cada vez
   * un nodo de la lista.
   * La lista de calles debe estar ordenada de mayor a menor precio de edificio.
   */
  void calculoUtilidadEsperadaConstruirImprimir ( long cantidad );

};

inline string jugador::get_nombre () const
{
  return _nombre;
}

inline string jugador::get_color () const
{
  return _color;
}

inline long jugador::get_dinero_total () const
{
  return _dinero_total;
}

inline long jugador::get_capital () const
{
  return _capital;
}

inline int jugador::get_turnosEncarcelado () const
{
  return _turnosEncarcelado;
}

inline int jugador::get_tiradasDobles () const
{
  return _tiradasDobles;
}

inline long jugador::get_estado () const
{
  return _estado;
}

inline casilla *jugador::get_estaEn () const
{
  return _estaEn;
}

inline int jugador::get_multiPago () const
{
  return _multiPago;
}

inline int jugador::get_numPropiedades () const
{
  return _posee.size ();
}

inline list <tarjeta*> jugador::get_tieneTarjeta () const
{
  return _tieneTarjeta;
}

inline void jugador::set_nombre ( string nombre )
{
  _nombre = nombre;
}

inline void jugador::set_color ( string color )
{
  _color = color;
}

inline void jugador::set_tiradasDobles ( int tiradasDobles )
{
  _tiradasDobles = tiradasDobles;
}

inline void jugador::set_estado ( long estado )
{
  _estado = estado;
}

inline void jugador::set_estaEn ( casilla *nuevaCasilla )
{
  _estaEn = nuevaCasilla;
}

inline void jugador::set_turnosEncarcelado ( int turnosEncarcelado )
{
  _turnosEncarcelado = turnosEncarcelado;
}

inline void jugador::incluir_tieneTarjeta ( tarjeta *tjt )
{
  _tieneTarjeta.push_front ( tjt );
}

inline void jugador::set_capital ( long capital )
{
  _capital = capital;
}

inline void jugador::set_dinero_total ( long dinero_total )
{
  _dinero_total = dinero_total;
}

inline void jugador::set_multiPago ( int multiPago )
{
  _multiPago = multiPago;
}

inline void jugador::set_tieneTarjeta ( list <tarjeta*> tieneTarjeta )
{
  _tieneTarjeta = tieneTarjeta;
}


#endif
