/***********************************************************************************
 *  CLASE JUGADOR_REGLAS
 *  Clase que incluye las reglas de los agentes de IA basados en reglas. Se distiguen
 *  distintos tipos de reglas segn el valor del atributo _reglas
 *
 *  Requiere:
 *     - clase jugador (herencia)
 *     - clase cadenas
 *     - clase mtrand
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __JUGADOR_REGLAS__
#define __JUGADOR_REGLAS__

#include "cadenas.h"
#include "jugador.h"

#include "mtrand.h"
#include <ctime>
using namespace std;

class jugador_reglas : public jugador {

 private:

  //Atributos
  int _reglas; //Define que tipo de reglas usa 

 public:

  // Constructor por defecto
  jugador_reglas ();
  
  // Constructor explcito
  jugador_reglas ( string nombre, string color, long dinero_total, long capital, int turnosEncarcelados, 
		   int tiradasDobles, long estado, int multiPago, int reglas );
 
  
  // Copy constructor
  jugador_reglas (const jugador_reglas& right);

  // Operador de asignacin
  jugador_reglas& operator= (const jugador_reglas& right);

  // Destructor
  ~jugador_reglas ();

  // Mtodos GET
  int get_reglas () const;   
  
  // Mtodos SET
  void set_reglas ( int reglas );

  //Otros mtodos
  
  /**
   * Polimorfismo de la clase jugador.
   * Mtodo que devuelve una cadena de caracteres sobre el jugador lista para ser guardada en un 
   * fichero de partida guardada. La informacin incluye nombre, color, dinero, capital, turnos 
   * encarcelado, tiradas dobles, estado, posicin actual y el atributo reglas. 
   */
  string guardarDatosPersonales ();

  /**
   * Polimorfismo de la clase jugador.
   * Mtodo por el cual un jugador paga una cantidad. Si la cantidad es 
   * positiva, el jugador pagar dinero, en caso de ser negativa, el jugador estar
   * cobrando dinero. Al pagar, se debe restar la cantidad del dinero total y del
   * capital. Si no tiene dinero suficiente se tratar de conseguir automticamente
   * mediante hipotecas y ventas de edificios. Si aun as no consigue el dinero suficiente
   * se declara como moroso.  Si no tiene dinero suficiente, se indica que es moroso (devuelve -1).
   */
  long pagar ( long cantidad );

  /**
   * Polimorfismo de la clase jugador.
   * El jugador decidir al caer sobre una propiedad libre si quiere comprarla o no. La decisin
   * se tomar en funcin del tipo de agente de IA que se trate.
   */
  bool decidirComprar ( propiedad *p );

  /**
   * Polimorfismo de la clase jugador.
   * El jugador decidir si salir de la crcel o no segn estime que convenga o no pagar la multa.
   * Es necesario conocer la multa y se pasa el argumento PropiedadesLibres para que el jugador basado en
   * reglas pueda decidir.
   */
  bool decidirSalirPagando ( int tpcPropiedadesLibres, long multa );

  /**
   * Polimorfismo de la clase jugador.
   * El jugador decidir si sale de la crcel segn estime conveniente o no. Slo es aplicable si tiene
   * tarjetas el jugador.
   */
  bool decidirSalirTarjeta ( int tpcPropiedadesLibres );

  /**
   * Polimorfismo de la clase jugador.
   * El jugador inteligente decidir que opcin es ms conveniente para cada momento en funcin 
   * de las reglas establecidas.
   */
  int decidirAccionTarjeta ( int numAcciones, tarjeta *tjt );

  /**
   * Polimorfismo de la clase jugador.
   * El jugador decidir tomar la opcin a pagar que menor gasto suponga
   */
  char decidirImpuestoEspecial ( long cantidad, int tpc );

  /**
   * Polimorfismo de la clase jugador.
   * El jugador decidir la nueva cantidad a pujar, si lo consider oportuno. El jugador basado en reglas
   * lo har en funcin de su razonamiento, y el aleatorio por puro azar.
   */
  long pujar( propiedad *p, long maxActual, jugador *ganador );

  /**
   * Mtodo que hipoteca una serie de propiedades del jugador con comportamiento aleatorio.
   * Hipotecar todas las propiedades desde la ms cara hasta la ms barata hasta que cubra
   * la deuda.
   */
  void hipotecarAleatorio ( long cantidad );

  /**
   * Algoritmo recursivo que busca la mejor alternativa de propiedades del jugador a hipotecar. Se debe conseguir
   * una cantidad de dinero superior a la variable cantidad. Las propiedades se introducen en el vector L. 
   * Index es la posicin a partir de la cual se comienza a buscar.
   */
  vector <propiedad*> alternativasHipoteca ( long cantidad, vector <propiedad*> L, int index );

  /**
   * Mtodo que busca las propiedades hipotecables y buscar la mejor alternativa para superar la cantidad 
   * minimizando las prdidas, usando el mtodo alternativasHipotecarImprimir.
   */
  vector <propiedad*> calculoUtilidadEsperadaHipotecar ( long cantidadNecesaria );
  
  /**
   * Mtodo para vender casas del jugador aleatorio. Vender todas las casas desde la propiedad ms cara hasta
   * la ms barata hasta superar la cantidad necesaria. As se asegura alcanzar el mximo dinero posible.
   */
  void venderAleatorio ( long cantidad );

  /**
   * Algoritmo recursivo que busca la mejor alternativa de para vender edificios de un jugador. Se debe conseguir
   * una cantidad de dinero superior a la variable cantidad. Las calles de donde se venden se introducen en el vector L. 
   * Index es la posicin a partir de la cual se comienza a buscar.
   */
  vector <calle*> alternativasVender ( long cantidad, vector <calle*> L, int index );

  /**
   * Mtodo que busca los edificios vendibles y buscar la mejor alternativa para superar la cantidad 
   * minimizando las prdidas, usando el mtodo alternativasVenderImprimir.
   * Una lista de edificios se representa como una lista de calles, cada aparicin de una calle, representa
   * un edificio.
   */
  vector <calle*> calculoUtilidadEsperadaVender ( long cantidad );

  /**
   * Llama a los metodos de deshipoteca en funcion del tipo de agente que se trate
   */
  void razonarDeshipotecas ();

  /**
   * Deshipoteca las propiedades de una forma aleatoria en funcin de unas probabilidades calculadas
   */
  void deshipotecarAleatorio ();

  /**
   * Algoritmo recursivo que busca la mejor alternativa de deshipotecas de propiedades. La cantidad mxima que se 
   * puede invertir es la cantidad que se marca en la variable 'cantidad'. La lista contiene todas las propiedades
   * hipotecadas ordenadas por precio de deshipoteca.
   */
  vector <propiedad*> alternativasDeshipotecar ( long cantidad, vector <propiedad*> L, int index );

  /**
   * Mtodo que busca las propiedades hipotecadas y buscar la mejor combinacin para deshipotecarlas sin
   * superar la cantidad indicada en el atributo cantidad. Utilizar el mtodo alternativasDeshipotecarImprimir.
   */
  vector <propiedad*> calculoUtilidadEsperadaDeshipotecar ( long cantidad );
  
  /**
   * Llama a los metodos de construccion en funcion del tipo de agente que se trate
   */
  void razonarConstrucciones ();
  
  /**
   * Construye sobre las calles construibles de una forma aleatoria en funcin de unas probabilidades calculadas
   * a partir del dinero total y el precio de construir
   */
  void construirAleatorio ();

  /**
   * Mtodo que busca todas las alternativas para construir de una lista de calles y devuelve la mejor alternativa.
   * Un edificio se representa en la lista por la calle en la que se construir.
   */
  vector <calle*> alternativasConstruir ( long cantidad, vector <calle*> L, int index );

  /**
   * Mtodo que usando el metodo alternativasConstruir busca la mejor alternativa para construir sin 
   * sobrepasar la cantidad indicada. Iterativamente buscar la mejor opcin indicando como inicio cada vez
   * un nodo de la lista.
   * La lista de calles debe estar ordenada de mayor a menor precio de edificio.
   */
  vector <calle*> calculoUtilidadEsperadaConstruir ( long cantidad );

};

inline int jugador_reglas::get_reglas () const
{
  return _reglas;
}

inline void jugador_reglas::set_reglas ( int reglas )
{
  _reglas = reglas;
}


#endif
