/***********************************************************************************
 *  CLASE NEGOCIACIÓN
 *  Clase que representa una negociación entre dos jugadores. 
 *  Una negociación puede incluir tarjetas y propiedades y debe darse entre dos y sólo
 *  dos jugadores.
 *
 *  Requiere:
 *     - clase propiedad ( jerarquia )
 *     - clase tarjeta
 *     - clase jugador
 *     - clase list (STL)
 *     - clase string (STL) 
 *
 *  Autor: Javier Aragón Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __NEGOCIACION__
#define __NEGOCIACION__

#include "propiedad.h"
#include "tarjeta.h"
#include "jugador.h"
#include <list>
#include <string>
using namespace std;

class negociacion {

 private: 
  
  //Atributos
  /**
   * Dinero que se intercambian
   */
  long _dineroEfectivo;
  long _dineroEfectivoPedido;
  
  /**
   * Intervienen en la negociacion un emisor y un receptor
   */
  jugador *_emisor;
  jugador *_receptor;

  /**
   * Propiedades y tarjetas que se ofrecen
   */
  list<propiedad*> _propiedadesOfertadas;
  list<tarjeta*> _tarjetasOfertadas;

  /**
   * Propiedades y tarjetas que se piden a cambio
   */
  list<propiedad*> _propiedadesPedidas;
  list<tarjeta*> _tarjetasPedidas;

 public:   
  // Constructores
  negociacion ();
  negociacion ( long dineroEfectivo,  long dineroEfectivoPedido, jugador *emisor, jugador *receptor, 
		list<propiedad*> propiedadesOfertadas, list<tarjeta*> tarjetasOfertadas, list<propiedad*> propiedadesPedidas,
		list<tarjeta*> tarjetasPedidas );
  // Copy constructor
  negociacion (const negociacion& right);

  // Operador de asignacion
  negociacion& negociacion::operator= (const negociacion& right);

  // Sobrecarga del operador de salida
  friend ostream& operator<< ( ostream &os, negociacion* n );
  
  //Destructor
  ~negociacion ();

  // Métodos GET
  long get_dineroEfectivo () const;
  long get_dineroEfectivoPedido () const;
  jugador* get_emisor () const;
  jugador* get_receptor () const;
  list<propiedad*> get_propiedadesOfertadas () const;
  list<propiedad*> get_propiedadesPedidas () const;
  list<tarjeta*> get_tarjetasOfertadas () const;
  list<tarjeta*> get_tarjetasPedidas () const;
  
  // Métodos SET
  void set_dineroEfectivo ( long dineroEfectivo );
  void set_dineroEfectivoPedido ( long dineroEfectivo );
  void set_emisor ( jugador *emisor );
  void set_receptor ( jugador *receptor );
  void set_propiedadesOfertadas ( list<propiedad*> propiedadesOfertadas );
  void set_propiedadesPedidas ( list<propiedad*> propiedadesPedidas );
  void set_tarjetasOfertadas ( list<tarjeta*> tarjetasOfertadas );
  void set_tarjetasPedidas ( list<tarjeta*> tarjetasPedidas );

  /**
   * Método que dirige una negociacion por el proceso normal:
   *  - Identificar el emisor
   *  - Identificar el receptor
   *  - Indicar lo que se ofrece
   *  - Indicar lo que se pide a cambio
   */
  int negociar ();
  
  /**
   * Seleccionar todo lo que se quiere ofrecer. Debe incluir tarjetas, propieades y dinero.
   */
  int ofrecer ();

  /**
   * Seleccionar todo lo que se quiere perdir a cambio en una negociación. Debe incluir tarjetas, 
   * propieades y dinero.
   */
  int pedir ();
  
  /**
   * Las propiedades incluidas en la oferta por el emisor son adjudicadas al receptor y viceversa
   */
  void distribuirBienes ();  
  
  /**
   * Método que lista todas las propiedades incluidas en una negociacion como oferta
   */
  string verPropiedadesOfertadas ();

  /**
   * Método que lista todas las propiedades incluidas en una negociacion como petición
   */
  string verPropiedadesPedidas ();

  /**
   * Comrpueba si la propiedad 'p' ha sido ya incluida en la oferta realizada
   */
  bool propiedadOfertadaIncluida ( propiedad *p );

  /**
   * Comrpueba si la propiedad 'p' ha sido ya incluida en la oferta realizada como petición
   */
  bool propiedadPedidaIncluida ( propiedad *p );

  /**
   * Método que permite al jugador introducir las propiedades que quiere añadir a la oferta.
   * Se muestra una lista y el jugador deberá seleccionar de esa lista aquellas propiedades que
   * quiera adjuntar identificandolas por su número de propiedad.
   */
  void incluirPropiedadesOfertadas ( );

  /**
   * Método que permite al jugador introducir las propiedades que quiere pedir al receptor..
   * Se muestra una lista y el jugador deberá seleccionar de esa lista aquellas propiedades que
   * quiera adjuntar identificandolas por su número de propiedad.
   */
  void incluirPropiedadesPedidas ( );

  /**
   * Método que prueba la factibilidad de incluir una propiedad en la negociación como oferta.
   * Comprobará que la calle no este construida y si lo está, deberá ofrecer incluir todo el monopolio
   * con casas y hoteles incluidas.
   * Finalmente, añadirá la propiedad a la lista de ofertadas.
   */
  int nuevaPropiedadOfertada ( propiedad *p );

  /**
   * Método que prueba la factibilidad de incluir una propiedad en la negociación como petición.
   * Comprobará que la calle no este construida y si lo está, deberá ofrecer incluir todo el monopolio
   * con casas y hoteles incluidas.
   * Finalmente, añadirá la propiedad a la lista de peticiones.
   */
  int nuevaPropiedadPedida ( propiedad *p );
  
  /**
   * Añade ordenadamente la propiedad p a la lista de propiedades ofertadas
   */
  void anyadirPropiedadOfertada ( propiedad *p );

  /**
   * Añade ordenadamente la propiedad p a la lista de propiedades pedidas
   */
  void anyadirPropiedadPedida ( propiedad *p );    
  
  /**
   * Añade una tarjeta a las tarjetas ofertadas. La recoge de la lista de tarjetas del emisor
   */
  void incluirTarjetasOfertadas ( int i );

  /**
   * Añade una tarjeta a las tarjetas pedidas. La recoge de la lista de tarjetas del receptor
   */
  void incluirTarjetasPedidas ( int i );

  /**
   * Insertar una tarjeta en la lista de tarjetas ofertadas
   */
  void anyadirTarjetaOfertada ( tarjeta *t );

  /**
   * Insertar una tarjeta en la lista de tarjetas pedidas
   */
  void anyadirTarjetaPedida ( tarjeta *t );
};

//GET:

inline long negociacion::get_dineroEfectivo () const
{
  return _dineroEfectivo;
}

inline long negociacion::get_dineroEfectivoPedido () const
{
  return _dineroEfectivoPedido;
}

inline jugador *negociacion::get_emisor () const
{
  return _emisor;
}

inline jugador *negociacion::get_receptor () const
{
  return _receptor;
}

inline list<propiedad*> negociacion::get_propiedadesOfertadas () const
{
  return _propiedadesOfertadas;
}

inline list<propiedad*> negociacion::get_propiedadesPedidas () const
{
  return _propiedadesPedidas;
}

inline list<tarjeta*> negociacion::get_tarjetasOfertadas () const
{
  return _tarjetasOfertadas;
}

inline list<tarjeta*> negociacion::get_tarjetasPedidas () const
{
  return _tarjetasPedidas;
}

//SET:

inline void negociacion::set_dineroEfectivo ( long dineroEfectivo )
{
  _dineroEfectivo = dineroEfectivo;
}

inline void negociacion::set_dineroEfectivoPedido ( long dineroEfectivo )
{
  _dineroEfectivoPedido = dineroEfectivo;
}


inline void negociacion::set_emisor ( jugador *emisor )
{
  _emisor = emisor;
}

inline void negociacion::set_receptor ( jugador *receptor )
{
  _receptor = receptor;
}

inline void negociacion::set_propiedadesOfertadas ( list<propiedad*> propiedadesOfertadas )
{
  _propiedadesOfertadas = propiedadesOfertadas;
}

inline void negociacion::set_propiedadesPedidas ( list<propiedad*> propiedadesPedidas )
{
  _propiedadesPedidas = propiedadesPedidas;
}

inline void negociacion::set_tarjetasOfertadas ( list<tarjeta*> tarjetasOfertadas )
{
  _tarjetasOfertadas = tarjetasOfertadas;
}

inline void negociacion::set_tarjetasPedidas ( list<tarjeta*> tarjetasPedidas )
{
  _tarjetasPedidas = tarjetasPedidas;
}


#endif
