/***********************************************************************************
 *  CLASE PROPIEDAD
 *  Clase que representa una propiedad del juego del monopoly. Es una clase derivada
 *  de la clase casilla y a su vez es una clase base de los distintos tipos de 
 *  propiedades del juego: calle, estacion y servicio.
 *
 *  Esta clase tiene como particularidad que se puede comprar y por lo tanto tendr
 *  un propietario.
 *
 *  Esta clase hereda de la clase casilla el nmero de casilla y el nombre y aporta
 *  como atributos el precio de la propiedad, su hipoteca y el estado hipotecado e
 *  hipotecable (ambos verdadero o falso)
 *
 *  Requiere:
 *     - clase casilla (herencia)
 *     - clase propietario (y derivadas jugador y banca)
 *     - clase monopolio
 *     - clases de la STL string y list
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/

#ifndef __PROPIEDAD__
#define __PROPIEDAD__

#include "propietario.h"
#include "jugador.h"
#include "banca.h"
#include "monopolio.h"
#include "casilla.h"
#include <string>
#include <list>
using namespace std;

class propiedad : public casilla {

 private:

  //Atributos
  long _precio;
  long _hipoteca;
  bool _hipotecado;
  bool _hipotecable;

 protected:  
  propietario *_esPropiedad;
  monopolio* _perteneceMonopolio;

 public:

  // Constructor por defecto
  propiedad ();
  
  // Constructor explcito
  propiedad ( int numero, string nombre, string nombreCorto, long precio, long hipoteca, bool hipotecado, bool hipotecable, 
	      propietario *p);
  
  // Copy constructor
  propiedad (const propiedad& right);

  // Operador menor
  bool operator< ( propiedad *compara );
  
  // Operador de asignacin
  propiedad& operator= (const propiedad& right);
  
  // Sobrecarga del operador de salida
  friend ostream& operator<< (ostream& os, propiedad* c);

  // Destructor
  ~propiedad ();

  // Mtodos GET
  long get_precio () const;
  long get_hipoteca () const;
  bool get_hipotecado () const;
  bool get_hipotecable () const; 
  propietario* get_esPropiedad () const;
  monopolio* get_perteneceMonopolio () const;
  long get_deshipoteca () const;


  // Mtodos SET
  void set_precio (long value);
  void set_hipoteca (long value);
  void set_hipotecado (bool value);
  void set_hipotecable (bool value);
  void set_esPropiedad (propietario* j);
  void set_perteneceMonopolio ( monopolio* perteneceMonopolio );

  // OTROS MTODOS
  
  /**
   * Mtodo que comprueba si el monopolio al que pertenece esta calle es el mismo que el
   * que se indica como parmetro (color)
   */
  bool esMonopolio ( string color );

  /**
   * Mtodo que crea un monopolio con el color pasado como parmetro. Se dar valor 1 al 
   * atributo numPropiedades de la clase monopolio porque si se crea el monopolio es porque
   * no exisita previamente y por lo tanto el nmero de propiedades actual es ese (1).
   */
  void crearMonopolio ( string color );

  /**
   * Mtodo que asigna una calle al mismo monopolio al que pertenece la calle pasada como
   * parmetro.
   */
  void asignarMonopolio ( propiedad* p );

  /**
   * Devuelve el nombre del monopolio al que pertenece.
   */
  string verColor ();
  
  /**
   * Comprueba si la propiedad tiene dueo o est disponible para ser comprada
   */
  bool disponible ();

  //Metodos virtuales: POLIMORFISMOS

  /**
   * Mtodo que muestra la informacin de una propiedad
   */
  virtual string mostrar ();

  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las filas de arriba y abajo del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres. En este caso se muestra la informacin propia de una propiedad.
   */
  virtual string dibujarCasillaHorizontal ( int fila, int anchoCasilla );
  
  /**
   * Mtodo que dibuja la fila indicada de la casilla correspondiente (en las columnas derecha e izquierda del tablero).
   * Se indicar la fila que se tiene que dibujar y el ancho con el que se tiene que pintar. Como resultado se
   * devolver una cadena de caracteres. En este caso se muestra la informacin propia de una propiedad
   */
  virtual string dibujarCasillaVertical ( int fila, int anchoCasilla );

  /**
   * Mtodo que devuelve una cadena de caracteres que representa como es la casilla en su 
   * imgen en el tablero, indicando aquellos datos de inters para un jugador que caiga 
   * sobre ella:
   *  -Numero
   *  -Nombre
   *  -Precio
   *  -Hipoteca
   */
  virtual string dibujarCasillaEnTablero (); 
  
  /**
   * Mtodo que calcula el alquiler de una propiedad
   */
  virtual long calcularAlquiler ( jugador *j );

  /**
   * Se adjudica la propiedad a un propietario genrico dado, indpendientemente de que se trate de 
   * la banca o de un jugador
   */
  virtual int adjudicarA ( propietario* j );

  /**
   * El jugador paga el alquiler que se ha calculado para la propiedad dada. Si el jugador no
   * tiene dinero quedar como un jugador moroso. El propietario de la propiedad cobrar dicha
   * cantidad.
   */
  int pagarAlquiler ( jugador* j );
  
  /**
   * Mtodo polimrfico de la clase casilla. En el caso de las propiedades, el mtodo tiene dos 
   * posibles actuaciones. Si la casilla tiene dueo, se debe pagar el alquiler al propietario.
   * Si no tiene dueo, se ofrece al jugador la posibilidad de comprar la propiedad. Si acepta,
   * pagar el precio y se aadir a su lista de propiedades, si no, la casilla ser subastada.
   */
  virtual int accion ( jugador *player );

  /**
   * El jugador compra la propiedad a cambio del precio establecido. Se descontar la cantidad de 
   * dinero del dinero total del jugador y se aadir dicha propiedad a su lista de propiedades.
   */
  virtual int comprar ( jugador* j );
  
  /**
   * Comrpueba si una propiedad tiene el mismo propietario que otra.
   */
  bool mismoPropietario ( propiedad* p );

  /**
   * Devuelve la informacin de una casilla en forma de cadena de caracteres (string).
   */
  virtual string infoString ();

  /**
   * Mtodo que hipoteca la propiedad. El propietario recibir la cantidad de dinero indicada en
   * ese caso y la propiedad pasar a tener estado hipotecado (y no hipotecable).
   */
  virtual int hipotecar ();

  /**
   * Mtodo que deshipoteca la propiedad, indicando dicho estado a la propiedad y cobrando al jugador la
   * cantidad indicada.
   */
  virtual int deshipotecar ();
  
  /**
   * Indica si una propiedad se puede ofertar en una negociacin
   */
  virtual bool esOfertable ();

  /**
   * Obtiene todas las propiedades que pertenecen al mismo monopolio
   */
  set <propiedad*> get_monopolioCompleto ( propiedad *p );

  /**
   * Asigna la propiedad al propietario p sin cobrar ni preguntar. 
   * Este propietario suele ser la banca.
   */
  virtual void liberar ( propietario *j);

  /**
   * Devuelve una cadena de caracteres con el formato necesario para guardarla en un fichero en el que
   * se est salvando una partida en curso
   */
  virtual string guardar();

  /**
   * Mtodo que indica si est completamente libre el monopolio al que pertenece la propiedad.
   */
  bool monopolioLibre ();

  /**
   * Mtodo que indica si esta es la ltima propiedad que le falta a el jugador pasado como argumento
   * para completar el monopolio
   */
  bool ultimaPropiedadMonopolio ( jugador *j );

  /**
   * Devuelve verdadero si el jugador j es el nico que tiene propiedades del monopolio al que pertence
   * la propiedad actual
   */
  bool unicoPoseedorMonopolio ( jugador *j );  
  
  /**
   * Mtodo que comprueba si el jugador j posee parte del monopolio al que pertence esta propiedad.
   */
  bool poseeParteMonopolio ( jugador *j );

  /**
   * Mtodo que indica si esta es la ltima propiedad que le falta a un jugador distinto al pasado 
   * como argumento para completar el monopolio.
   */
  bool ultimaPropiedadMonopolioOtroJugador ( jugador *j );

};

inline long propiedad::get_precio () const
{
  return _precio;
}

inline long propiedad::get_hipoteca () const
{
  return _hipoteca;
}

inline bool propiedad::get_hipotecable () const
{
  return _hipotecable;
}

inline bool propiedad::get_hipotecado () const
{
  return _hipotecado;
}

inline propietario* propiedad::get_esPropiedad () const
{
  return _esPropiedad;
}

inline monopolio* propiedad::get_perteneceMonopolio () const
{
  return _perteneceMonopolio;
}
 
inline long propiedad::get_deshipoteca () const
{
  return ( _hipoteca * ( 12 / 10 ) );
}

inline void propiedad::set_precio (long value)
{
  _precio = value;
}

inline void propiedad::set_hipoteca (long value)
{
  _hipoteca = value;
}

inline void propiedad::set_hipotecado (bool value)
{
  _hipotecado = value;
}

inline void propiedad::set_hipotecable (bool value)
{
  _hipotecable = value;
}

inline void propiedad::set_esPropiedad ( propietario* j )
{
  _esPropiedad = j;
}

inline void propiedad::set_perteneceMonopolio ( monopolio* perteneceMonopolio )
{
  _perteneceMonopolio = perteneceMonopolio;
}

#endif
