/***********************************************************************************
 *  STREET MASTER'S
 *  Street Master's es un juego similar al popular juego del monopoly que permite 
 *  participar hasta 6 jugadores en una partida. Ha sido desarrollado bajo el S.O.
 *  Linux y cuenta con el uso de la librera GNU Readline, lo cual enriquece en 
 *  gran medida la apariencia de la consola. Para compilar Street Master's se debe
 *  usar el Makefile que se proporciona.
 *
 *  Este fichero de C++ incluye slo los comandos y las funciones relacionadas con 
 *  ellos. Las funcionalidades del juego se resuelven en las llamadas a las clases
 *  necesarias en cada momento.
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/
/*	Constantes	*/

//Librerias propias
#include "configuracion.h"
#include "cadenas.h"
#include "fichero.h"

//Librerias propias (clases)
#include "tablero.h"

//Libreria GNU Readline
#include <readline/readline.h>
#include <readline/history.h>

//librerias de C / C++
#include <stdlib.h>     //getenv
#include <ctype.h>	//toupper
#include <unistd.h>	//fork, exec
#include <sys/wait.h>	//wait
#include <iostream>	//entrada y salida
#include <string>	//Clase string de C++
#include <vector>       //Clase vector de C++
using namespace std;



/**
 * DEFINICION DE TIPOS
 */

/**
 * Tipo COMANDO, es la informacin necesaria para cada comando 
 */
typedef struct {  
  int id;		// Identificador entero del comando
  char *nombre;	        // Nombre del comando
} COMANDO;

/**
 * Lista de comandos de street_master's. Especificado as para usar la librera readline
 */
COMANDO comandos[] = 
{
  { 0, "salir" },
  { 1, "nuevo" },
  { 2, "ayuda" },
  { 3, "lista" },
  { 4, "ver" },
  { 5, "cerrar" },
  { 6, "tirar" },
  { 7, "fin" },
  { 8, "bancarrota" },
  { 9, "pagar" },
  { 10, "cambiar" },
  { 11, "hipotecar" },
  { 12, "deshipotecar" },
  { 13, "construir" },
  { 14, "vender" },
  { 15, "negociar" },
  { 16, "guardar" },
  { 17, "cargar" },
  { 18, "minicomo" },
  { 19, "history" },
  { 20, "uEsperada" },
  { 21, "log" },  
  { -1, "ls" },
  { -1, "cat" },
  { -2, "rm" },
  { -2, "make" },
  { -2, "as" }
};

// Nmero total de comandos que tiene street master's 
// Si hay 7 comandos, el valor de total_comandos ser 6 (0..6) 
static const int total_comandos = 26;
static const string nombreJuego = "street master's";
static const int MaxJugadores = 6;

/**
 * Variable esttica para cargar la lnea 
 */
static char *line_read = ( char * ) NULL;


/**********FUNCIONES TILES PARA EL TRATAMIENTO DE CADENAS**************/


/**
 * Obtiene los argumentos dados en la cadena de caracteres 'linea' y los almacena en el 
 * array de cadenas 'argumentos', haciendo que la ltima posicin del array apunte a NULL
 * para pasarlo como argumento de execvp.   
 *  IN:	max:	mxima capacidad del array. Si hay ms argumentos que max, se truncar en max
 *	linea:	string de la que se quieren separar los argumentos
 *  OUT:argumentos:	array con un argumento en cada posicion
 *  	return:		nmero de argumentos
 */
int obtener_argumentos ( char *linea, char **argumentos, int max )
{
  int i=0;
  int contador_args = 0;
  //si se inicia desde 0 el array	
	
  while ( ( linea[i]!='\0' ) && ( contador_args < max ) )
    {
      argumentos[contador_args] = new char[20];
      //Elimina espacios al principio
      while ( linea[i] && whitespace ( linea[i] ) )
	i++;
      //copia el argumento correspondiente	
      int pos=0;
      while ( linea[i] && !whitespace ( linea[i] ) )
	{
	  argumentos[contador_args][pos++]=linea[i];
	  i++;
	}
      //Elimina espacios al final
      while ( linea[i] && whitespace ( linea[i] ) )
	i++;
      //fin de palabra (argumento)
      argumentos[contador_args][pos]='\0';		
      contador_args++;	//siguiente argumento		
    }
  argumentos[contador_args] = new char[20];
  argumentos[contador_args] = NULL;
  return contador_args;
}


/**
 * Libera toda la memoria de un array de cadenas de caracteres. Elimina tambin
 *  el array de la memoria
 *  IN:	argumentos: array de cadenas que se quiere liberar
 *  	max:	posiciones del array
 */
void liberar_memoria(char **argumentos, int max)
{
  for (int k=0; k<=max; k++)
    {
      delete[] argumentos[k];
      argumentos[k]=NULL;
    }
	
  delete[] argumentos;
  argumentos = NULL;
}



// FUNCIONES PARA EL USO READLINE



/**
 * Funcin que genera el comando necesario cuando se presiona 'tab'.
 * STATE nos permite conocer en que punto de la lista se debe comenzar
 * a buscar. Si STAT = 0, se comenzar el principio de la lista 
 */
char *generar_comando (const char *text, int state)
{
  static int list_index, len;
  char *name;
	
  // Si se trata de una palabra nueva para completar -> inicializar.
  // Por eficiencia se salvara la longitud de 'texto', y se inicializar
  // 'index' = 0 
  if (!state)
    {
      list_index = 0;
      len = strlen (text);
    }
	
  // Devuelve el siguiente comando que coincide parcialmente en la lista 
  // de comandos
  while ( ( name = comandos[list_index].nombre ) && ( list_index < total_comandos ) )
    {		
      list_index++;
      if (strncmp (name, text, len) == 0)
	return (dupstr(name));
    }
  // Si no hay coincidencias, devuelve NULL
  return ((char *)NULL);
}


/**
 *  Intentar completar el contenido de 'text'.  START y END marcan la 
 *  region de rl_line_buffer que contiene la palabra a completar. TEXT es
 *  la palabra a completar. Devuelve el array de coincidencias, o NULL si
 *  no hay ninguna 
 */
char **sm_completion (const char *text, int start, int end)
{
  char **matches;

  matches = (char **)NULL;
  // Si esta palabra esta al inicio de la linea, es un comando a 
  // completar, si no es el nombre de un directorio
  if (start == 0)
    matches = rl_completion_matches (text, generar_comando);
  return (matches);
}


/**
 * Inicializa los valores de readline
 */
void initialize_readline ( void )
{
  rl_attempted_completion_function = sm_completion;
}


/**
 *  Lee y devuelve un string usando la funcin readline.
 *  Adems aade al historial las lineas introducidas.
 *  Si es EOF devuelve NULL 
 */   
char *rl_gets ( string prompt )
{  
  // Si el buffer ya esta ocupado, devuelve la memoria 
  // al estado de libre 
  if ( line_read )
    {
      delete ( line_read );
      line_read = ( char * ) NULL;		
    }
	
  // Obtiene una linea del usuario
  line_read = readline ( prompt.c_str() );
  //Si la linea contiene algo, la guarda en el historial
  if ( line_read && *line_read )
    add_history ( line_read );
		
  return ( line_read );
}



// METODOS PARA SELECCION DE COLOR DE LAS FICHAS

/**
 * Mtodo que inicializa un vector de strings con los colores de las fichas del juego
 */
void inicializar_colores ( vector <string> &fichas )
{
  fichas.push_back ( "amarillo" );
  fichas.push_back ( "rojo" );
  fichas.push_back ( "azul" );
  fichas.push_back ( "verde" );
  fichas.push_back ( "naranja" );
  fichas.push_back ( "blanco" );
}

/**
 * Mtodo que muestra los colores de fichas disponibles en el tablero.
 */
void mostrar_colores_disponibles ( vector <string> fichas )
{
  for ( int i=0; i != fichas.size(); i++ )    
      cout << fichas[i] << " ";
}


/**
 *  Comprueba que el color elegido es vlido. Si el color es vlido, se
 *  pondr como no disponible para futuros accesos. Es valido si est en la lista
 *  de colores disponibles (introducida en el argumento fichas). Una vez comprobado, 
 *  se elimina el color del vector porque ya no es disponible.
 */
bool validar_color ( vector <string> &fichas, string color )
{
  for ( int i=0; i != fichas.size(); i++ )	
    if ( fichas[i] == color )
      {
	//Lo elimina porque ya esta asignado
	fichas.erase ( fichas.begin() + i );			
	return true;
      }
  return false;
}

/**
 *  Elegir el primer color vlido del vector ficha para un jugador 
 */
string color_por_defecto ( vector <string> &ficha )
{
  string color;
  if ( !ficha.empty() )
    {
      color = ficha[0];
      //Lo elimina porque ya esta asignado
      ficha.erase ( ficha.begin() );
    }
  else
    color = "no color";

  return color;
}



// IMPLEMENTACIN DE LOS COMANDOS DEL JUEGO

/**
 *  Ejecuta cualquier comando simple de linux, aunque no este dentro de la lista de 
 *  posibles comandos (no se autocompletarn), se introduce el array de argumentos
 *  y se devolver -1 si no se ha podido ejecutar
 */
int com_linux( char **argumentos )
{
  pid_t pid;
  int status;
	
  //Ejecuta el comando
  //Crea un proceso hijo
  pid = fork();
  if (pid == 0)	//hijo
    {
      //El proceso hijo ejecuta el comando
      execvp(argumentos[0],argumentos);
      cerr << "No se ha podido ejecutar: " << argumentos[0] << endl;
      return -1;
    }
  else if (pid == -1)	//error
    {
      cerr << "No se ha podido ejecutar: " << argumentos[0] << endl;	
      return -1;
    }	
  else			//padre
    while (wait(&status) != pid);	//El padre espera la ejecucin del hijo
  
  return 0;	
}


/** 
 *  Metodo que sirve para elegir el nmero de jugadores. El metodo iterar hasta que 
 *  encuentre un valor correcto. Si al tercer intento el usuario no ha encontrado un
 *  valor correcto, devolver un error 
 */
int elegir_numero_jugadores ( tablero &partida )
{
  int ok = -1, intento = 0;
  int numJugadores = 0;
  //Repite el proceso hasta que el valor sea correcto
  do
    {
      //lectura de nmero de jugadores
      intento++;
      cout << "  Nmero de jugadores (2 - 6): "; 
      cin >> numJugadores;	
      if ( !((numJugadores >= 2) && (numJugadores <= 6 )) )
	{
	  cerr << "Nmero de jugadores no vlido.\n";	
	  ok=-1;
	  cin.clear(); //limpio las banderas de cin. Colocarlas en estado de bueno
	  cin.ignore( 2000, '\n' ); // vacia el buffer de la entrada ya que contiene "basura"
	}
      else{
	ok=0;
	partida.set_numJugadores ( numJugadores );
      }
    }while ( (( ok == -1 ) || ( cin.fail() )) && ( intento < 3 ) );

  return ok;
}

/** Metodo que crea los jugadores que indica la variable numJugadores.
 *  Pedir al usuario los datos pertinentes y enviando un mensaje a la
 *  clase tablero, lo creara e incluira en el tablero. Es evidente que 
 *  un jugador siempre tiene que estar asignado a un tablero. En el caso
 *  de que el jugador no introduzca su nombre o color (slo un retorno
 *  de carro) se le asignar uno por defecto 
 */
int crear_jugadores ( tablero &partida  )
{  
  string nombre, color; 	//datos que leo desde la pantalla
  bool color_ok;
  int iaux;  
  vector <string> fichas;
  inicializar_colores ( fichas );

  cout << "-Introduzca los siguientes datos para crear los jugadores:" << endl;
  cin.clear(); //limpio las banderas de cin. Colocarlas en estado de bueno
  cin.ignore( 2000, '\n' ); // vacia el buffer de la entrada ya que contiene "basura"
  
  //Itera preguntando por cada jugador
  int numJugadores = partida.get_numJugadores();
  for ( int id=0; id < numJugadores; id++ )
    {
      int jugadorAuto = 0;
      cout << "JUGADOR " << ( id + 1 ) << endl;
      cout << "Quieres que sea un jugador controlado por el ordenador? (s/N) ";      	
      char automatico;
      automatico = getchar();
      cin.clear();
      cin.ignore( 2000, '\n' );
      if ( automatico == 's' || automatico == 'S' )
	{
	  //Si es un jugador automtico debe indicar que tipo de jugador es.
	  //Repite el proceso hasta que sea correcta la respuesta
	  do 
	    {
	      cout << "Elije contra que tipo de jugador quieres jugar: " << endl;
	      cout << " 1 - Jugador inteligente" << endl;
	      cout << " 2 - Jugador con comportamiento aleatorio" << endl;
	      cout << "Opcin: ";
	      string opcion;
	      cin >> opcion;
	      jugadorAuto = atoi ( opcion.c_str() );
	      cin.clear();
	      cin.ignore(2000, '\n');
	      if ( jugadorAuto < 1 || jugadorAuto > 2 )
		cerr << endl << "Opcin no vlida" << endl ;
	    }
	  while ( jugadorAuto < 1 || jugadorAuto > 2 );	  

	  //Nombre del jugador automtico
	  char idAlpha[5];
	  sprintf( idAlpha, "%d", id + 1 );
	  if ( jugadorAuto == 1 )
	    nombre = "Jugador Inteligente #" + string ( idAlpha ); 
	  else 
	    nombre = "Jugador Aleatorio #" + string ( idAlpha ); 
	  cout << " -Nombre : " << nombre << endl;

	  //Color del jugador automtico
	  color = color_por_defecto ( fichas );
	  cout << " -Color ( asignado por defecto ): " << color << endl;
	}

      //Jugadores humanos
      else
	{
	  //NOMBRE
	  cout << " -Nombre: ";
	  getline ( std::cin, nombre ); //Lee el nombre del jugador (valen espacios)
	  while ( partida.repetido ( nombre ) )
	    {
	      cout << " - Nombre repetido, introduzca uno nuevo: ";
	      getline ( std::cin, nombre ); //Lee el nombre del jugador (valen espacios)
	    }

	  // Si no introducen ningn nombre se le da uno por defecto
	  if ( nombre == "" )
	    {
	      char idAlpha[5];
	      sprintf( idAlpha, "%d", id + 1 );
	      nombre = "Jugador #" + string ( idAlpha ); 
	      cout << " -Nombre ( asignado por defecto ): " << nombre << endl;
	    }  
	  
	  //COLOR
	  int intento = 0;
	  //repite hasta que sea correcto o haya 3 intentos
	  do
	    {
	      cout << " -Color ( ";
	      mostrar_colores_disponibles ( fichas );
	      cout << "): ";
	      getline ( std::cin, color ); //Lee el nombre del jugador (valen espacios)
	      intento++;
	      color_ok = validar_color ( fichas, color );
	      if ( (color_ok == false ) && ( color != "" ) )
		cerr << "ERROR: Color no vlido, elija uno de la lista" << endl;
	    }
	  while ( ( ( color_ok == false ) && ( intento < 3 ) ) && ( color != "" ) );
	  
	  // Si no ha elegido un color vlido se pone uno por defecto
	  if ( ( color_ok = false ) || ( color == "" ) )
	    {
	      color = color_por_defecto ( fichas );
	      cout << " -Color ( asignado por defecto ): " << color << endl;
	    }
	}
      cout << endl;

      //incluir el jugador en la partida
      partida.incluir_jugador( id, nombre, color, jugadorAuto );      
    } // for
  
  return 0;
}


/** 
 *  Mtodo que lee el fichero de OPCIONES y obtiene los datos necesarios para la carga del juego. Obtiene
 *  el nombre de los ficheros donde se encuentran los datos de:
 *   -tablero
 *   -suerte
 *   -caja de comunidad 
 */
int cargar_opciones ( tablero &partida, string fichero )
{
  //Abre el fichero de configuracion
  fstream ifs ( fichero.c_str (), ios::in );
  if ( !ifs )
    {
      cerr << "Fichero de configuracin no disponible. Compruebe que exista " << fichero << endl;
      return -1;
    }
  
  //leer datos del fichero  
  const int N = 100;
  string linea;
  int ncars=0, i;
  string::size_type pos;  
  int casasDisponibles = -1, hotelesDisponibles = -1;
  do
    {      
      getline ( ifs, linea, '\n' );     //lee una linea
      if ( ifs.eof() ) break;		//fin de fichero
      // Tratamos la linea leida
      
      // Elimina espacios en blanco y comentarios de una linea
      i=0;
      while ( ( linea[i] != '#') && (i < linea.size() ) )
        i++;

      // Limpia los comentarios detectados y los espacios en blanco 
      linea = linea.substr ( 0, i );
      linea = stripwhite_st ( linea );
      
      // Carga la ruta indicada en el fichero de configuracion en la variable correspondiente
      // Fichero de casillas
      if ( linea.substr ( 0, 7 ) == "TABLERO" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  partida.set_fichCasillas ( aux );
	}

      //Fichero de tarjetas de suerte
      if ( linea.substr ( 0, 6 ) == "SUERTE" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  partida.set_fichSuerte ( aux );
	}

      // Fichero de tarjetas de caja de comunidad
      if ( linea.substr ( 0, 14 ) == "CAJA COMUNIDAD" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  partida.set_fichCajaComunidad ( aux );
	}

      //Moneda de juego
      if ( linea.substr ( 0, 6 ) == "MONEDA" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  partida.set_moneda( aux );
	}

      //Dinero inicial para comenzar a jugar
      if ( linea.substr ( 0, 14 ) == "DINERO INICIAL" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );	  
	  partida.set_dineroInicial( atol ( aux.c_str() ) );
	}
      
      //Casas disponibles en la banca
      if ( linea.substr ( 0, 17 ) == "CASAS DISPONIBLES" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  casasDisponibles = atoi ( aux.c_str() );
	}

      //Hoteles disponibles en la banca
      if ( linea.substr ( 0, 19 ) == "HOTELES DISPONIBLES" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  hotelesDisponibles = atoi ( aux.c_str() );
	}

      //Indica si se permiten las subastas o no
      if ( linea.substr ( 0, 8 ) == "SUBASTAS" )
	{
	  pos = linea.find ( "=", 0 );
	  string aux = stripwhite_st ( linea.substr ( pos+1, linea.size () ) );
	  partida.set_subastas ( atoi ( aux.c_str() ) );
	}

      if ( ( ncars = ifs.gcount() ) == N-1 ) //se activo el failbit
	ifs.clear();
    } while ( true );
  
  if ( !ifs.eof() && ifs.fail() )
    {
      cerr << "Error al leer el fichero de ayuda\n";
      return -1;
    }
  if ( partida.incluirBanca ( casasDisponibles, hotelesDisponibles ) == -1 )
    cerr << "Se ha asignado el valor por defecto de hoteles y casas disponibles" << endl;
  ifs.clear();
  return 0;
  
}

/**
 *  Comando que crea una nueva partida. No estar disponible si existe una partida abierta   
 *  Se deja abierta la posibilidad de que se pase por linea de comando el nmero de jugadores
 *  o el nombre de los mismos
 */
int com_nuevo ( int num_args, tablero &partida, string &prompt )
{
  int intento = 0;
  int numJugadores, ok;
  string fich_tablero, fich_caja, fich_suerte, moneda;
 
  //si no existen argumentos, solo el comando
  if ( num_args == 1 )
    {
      cout << "Desea cargar la configuracin por defecto del juego?(recomendado) (s/n)";
      char opcion = cin.get();
      
      // Carga configuracion por defecto
      if ( ( opcion != 'n' ) && ( opcion != 'N' ) )
	{
	  string fichero = archivo_configuracion;
	  ok = cargar_opciones ( partida, fichero );
	  /* Opciones cargadas correctamente	*/
	  if ( ok != 0)
	    {
	      cout << "ERROR cargando el fichero de conficuracion " << archivo_configuracion << endl;	  
	      return -1;
	    }
	}
      // Pide los datos para cargar una configuracion alternativa
      else
	{
	  string fich_tablero, fich_suerte, fich_caja, moneda, sDineroInicial, sCasasDisponibles, sHotelesDisponibles;
	  string sSubastas;
	  cin.clear(); //limpio las banderas de cin. Colocarlas en estado de bueno
	  cin.ignore( 2000, '\n' ); // vacia el buffer de la entrada ya que contiene "basura"
	  cout << "Introduzca:" << endl;
	  //Fichero de casillas
	  cout << "  -Fichero para cargar el tablero: ";
	  getline ( std::cin, fich_tablero );
	  partida.set_fichCasillas ( fich_tablero );

	  //Fichero de tarjetas de suerte
	  cout << "  -Fichero para cargar el mazo de suerte: ";
	  getline ( std::cin, fich_suerte );
	  partida.set_fichSuerte ( fich_suerte );
	  
	  //Fichero de tarjetas de caja de comunida
	  cout << "  -Fichero para cargar el mazo de caja de comunidad: ";
	  getline( std::cin, fich_caja );
	  partida.set_fichCajaComunidad ( fich_caja );

	  //Moneda de juego
	  cout << "  -Tipo de moneda para jugar: ";
	  getline( std::cin, moneda );
	  partida.set_moneda ( moneda );

	  //Dinero inicial
	  cout << "  -Dinero inicial: ";
	  getline( std::cin, sDineroInicial );
	  partida.set_dineroInicial ( atol( sDineroInicial.c_str() ) );

	  //Casas disponibles inicialmente
	  cout << "  -Casas disponibles inicialmente: ";
	  getline( std::cin, sCasasDisponibles );
	  
	  //Hoteles disponibles inicialmente
	  cout << "  -Hoteles disponibles inicialmente: ";
	  getline( std::cin, sHotelesDisponibles );

	  //Crear la banca en el tablero
	  if ( partida.incluirBanca ( atoi( sCasasDisponibles.c_str() ), atoi( sHotelesDisponibles.c_str() ) ) == -1 )
	    cerr << "Se ha asignado el valor por defecto de hoteles y casas disponibles" << endl;
	  
	  //Preguntar sobre las subastas
	  cout << "Quieres que se subasten las propiedades que no se compren? (s/N)";
	  getline( std::cin, sSubastas );
	  if ( sSubastas == "s" or sSubastas == "S" )
	    partida.set_subastas ( true );
	  else
	    partida.set_subastas ( false );
	  ok = 0;
	}

      //Si todo es correcto, se procede a la creacion de la partida
      if ( ok == 0 )
	{	
	  //Cargar las tarjetas de suerte
	  ok = partida.crearMazo ( tarjetasSuerte );
	  if ( ok != 0 )	 
	    cout << "ERROR cargando las tarjetas de 'suerte' desde " << partida.get_fichSuerte () << endl;
	  if ( ok == 0 )
	    {
	      //Crear las tarjetas de caja de comunidad
	      ok = partida.crearMazo ( tarjetasCaja );
	      if ( ok != 0 )	
		cout << "ERROR cargando las tarjetas 'caja de comunidad' desde " << partida.get_fichCajaComunidad () << endl;
	    }	  
	  if ( ok == 0 )
	    {
	      //Cargar las casillas del tablero
	      ok = partida.cargar_casillas ();
	      //Si no se ha encontrado ninguna casilla de salida, se asigna una por defecto (la casilla 0)
	      if ( partida.get_salida () == -1 && ok == 0 )
		{
		  cerr << "No se ha encontrado una casilla de salida en el juego." << endl;
		  cerr << "Por defecto se asigna la casilla 0 como casilla de salida" << endl;		  
		  partida.set_salida ( 0 );
		  cout << "Pulse 'intro' para continuar";
		  getchar();
		}
	      if ( ok != 0 )
		cout << "ERROR cargando el tablero desde " << partida.get_fichCasillas () << endl;	     
	    }
	}
      //Si la creacion del tablero y las tarjetas ha sido correcta se crearn los jugadores
      if ( ok == 0 )
	{
	  //Intoducir el numero de jugadores
	  cout << endl << "Introduzca los siguientes datos para generar su partida" << endl;
	  ok = elegir_numero_jugadores ( partida );
	  if ( ok == 0 )
	    {
	      //Introducir jugadores en el tablero
	      ok = crear_jugadores ( partida );	      
	    }
	}      
      // Si se ha creado todo bien, se colocan los jugadores en la casilla de salida
      if ( ok == 0 )
	{
	  int i = 0;
	  while ( ( i < partida.get_numJugadores () ) && ( ok == 0 ) )
	    {
	      partida.colocarJugador ( i++, partida.get_salida () );
	    }
	  cout << endl << endl << "\t\tVAMOS A VER QUIEN EMPIEZA A JUGAR: " << endl << endl;
	  //Elegir el orden de tirada lanzando los dados
	  prompt = nombreJuego + " (" + partida.elegirOrdenTirada() + ")> ";
	  cout << endl << endl;
	}
      // Si se ha producido algn error se debe borrar todo lo creado
      if ( ok == -1 )
	{
	  cerr << "Restableciendo el sistema...";
	  partida.cerrar ();
	  //cerr << "HECHO" << endl;
	}
    }
  //si existen argumentos
  else
    {
      cerr << "No se esperan parmetros para el comando 'nuevo'\n";
      ok = -1;
    }
  
  return ( ok );
}

/**
 * Ejecuta el comando 'ver'. Este comando interpreta las siguientes opciones:
 *   - '-jugador': Muestra la informacin sobre todos los jugadores
 *   - '-jugador <color>': Muestra informacion sobre el jugador indicado
 *   - '-leyenda': Muestra la leyenda del juego
 *   - '-banca': Muestra el estado de banca
 *   - '-casilla <numCasilla>': Muestra la casilla indicada
 * El comando ver sin parmetro muestra el tablero.
 */
int com_ver ( char **argumentos, int num_args, tablero &partida )
{
  int ok;

  string nombre, color;
  int turno;
  long  dinero_total, capital;
  bool encontrado;

  //Si no se introduce una opcion muestra el tablero.
  if ( num_args == 1 )
    {
      cout << partida;
      ok = 0;
    }
  else
    {
      //Ver jugador
      if ( strcmp ( argumentos[1], "-jugador" ) == 0 )
	{
	  // Si no se especifica jugador se muestra la info de todos los jugadores 
	  if ( num_args == 2 )
	    ok = partida.ver_lista_jugadores ();    
	    
	  // Si se especifica que jugador se quiere ver, se muestra slo ese. Se debe especificar su color	    
	  else	    
	    ok = partida.ver_jugador ( argumentos[2] );
	}
      else 
	{
	  //Muestra la leyenda del juego
	  if ( ( strcmp ( argumentos[1], "-leyenda" ) == 0 ) && ( num_args == 2 ) )
	    {
	      cout << partida.leyenda();
	      ok = 0;
	    }
	  //Muestra los datos de la banca
	  else if ( ( strcmp ( argumentos[1], "-banca" ) == 0 ) && ( num_args == 2 ) )
	    {
	      partida.verBanca();
	      ok = 0;
	    }
	  //Muestra una casilla. Se debe indicar el numero de casilla
	  else if ( ( strcmp ( argumentos[1], "-casilla" ) == 0 ) )
	    if ( num_args == 3 )
	      {
		//Deteccion de errores al introducir el numero
		char* error;
		int num = strtol ( argumentos[2], &error, 10 );
		if ( ( argumentos[2] != NULL ) && ( strcmp(error,"") == 0 ) && ( num >= 0) )		
		  partida.verCasilla( num );
		else
		  {
		    cerr << "Argumento incorrecto: " << argumentos[2] << endl;
		  }
	      }
	    else
	       {
		 cerr << "Error: " << "Se espera un argumento para ver -casilla [numero]" << endl;
		 ok = -1;
	       }
	  else
	    {
	      cerr << "Error: " << argumentos[1] << " desconocido" << endl;
	      ok = -1;
	    }
	}
    } // else 
  return ( ok );
}


/**
 * Ejecuta el comando 'ls' de linux con los argumentos deseados 
 * OPCIONES: 
 * -Si no se introducen argumentos se realizar un listado de los ficheros/directorios
 *   de Street Master's
 * -Si el argumento es '-g', slo se mostrarn las partidas guardadas
 * Devuelve -1 si error 
 */
int com_lista (char **argumentos, int num_args)
{	
  pid_t pid;
  int status;
  
  if ( num_args <= 2 )
    {
      //Si se pasa algun parmetro, debe ser '-g', en caso contrario ser error
      if (num_args == 2)
	if ( strcmp ( argumentos[1], "-g" ) != 0 ) 
	  {
	    cerr << "Opcin no vlida para el comando 'lista'" << endl;
	    return ( -1 );
	  }      
      //Ejecucin del comando ls
      pid = fork();
      if ( pid == 0 )
	{	
	  //Si se introduce el parametro -g se muestran solo las partidas guardadas.
	  //'guardado' es una constante que se encuentra en el fichero configuracion.h
	  if (num_args == 2 )
	    {
	      if ( strcmp ( argumentos[1],"-g" ) == 0 )
		{
		  cout << endl << "PARTIDAS GUARDADAS: " << endl;
		  cout << "------------------- " << endl;
		  execlp ( "ls", "ls", guardado, NULL );
		}
	    }
	  //Si no hay opciones ejecuta un ls recursivo por todos los ficheros de Street Master's
	  else
	    execlp ("ls", "ls", "-R", NULL );
	  return -1;
	}
      else if ( pid == -1 )
	{
	  cerr << "No se a podido ejecutar: 'lista'.\n";
	  return -2;
	}	
      else
	while ( wait ( &status ) != pid );	//El padre espera la ejecucin del hijo
    }
  else
    {
      cerr << "Demasiados argumentos para el comando 'lista'" << endl;
      return -2;
    }
  return 0;	
}

/**
 *  Muestra el fichero de ayuda 
 *    -Si no se pasa ningn valor como parmetro, se mostrar el fichero que
 *    ensea el listado de comandos.
 *    -Si se pasa un parmetro se muestra el fichero que se corresponde con la
 *    ayuda pedida. 
 */
int com_ayuda ( char **argumentos, int num_args )
{  
  int ok;
  char *aux;	//auxiliar para concatenar la ruta
  
  //Limpia la pantalla
  system ( "clear" );
  if ( num_args <= 2 )
    {
      aux = new char[50];
      strcpy(aux,dir_ayuda);	//dir_ayuda: constante por eso lo copio a string
      //Si no hay argumentos ( comando = 'ayuda' ), muestra el fichero general de ayuda
      if ( num_args == 1 )
	ok = mostrar_fichero(strcat(aux,"help.txt"));
      //si existen argumentos, se muestra el fichero concreto con la ayuda pedida
      else
	ok = mostrar_fichero(strcat(aux,argumentos[1]));	
      delete[] aux;
    }
  else 
    ok = -1;
  return ok;
}

/**
 * Ejecuta el comando salir de street master. Si se pasa algn argumento
 * al comando, este dar error e informar de ello devolviendo -1, en caso
 * de ser una llamada correcta devolver 0. Se ofrece la oportunidad de guardar
 * antes de salir.
 */
int com_salir ( int num_args, tablero &partida )
{
  if ( num_args > 1 )
    {
      cerr << "Error: No se esperan argumentos para el comando 'salir'."<< endl;
      return -1;
    }
  else
    {
      int ok = 0;
      //Si hay una partida abierta, se guardarn los logs y se  ofrece la oportunidad de guardar
      //la partida antes de salir.
      if ( partida.get_partidaAbierta () )
	{
	  partida.guardarLog ();
	  cout << "Quieres guardar antes de salir? (s/N)";
	  char opcion = cin.get ();
	  if ( opcion == 's' || opcion == 'S' )
	    {
	      string aux;
	      cout << "Indica el nombre con el que se guardar la partida: ";
	      cin.clear();
	      cin >> aux;      
	      //Guardado es una cte que esta en configuracion.h
	      string fichero = guardado + aux + ".sav";
	      ok = partida.guardar ( fichero );
	    }
	  if ( ok == -1 )
	    cerr << "No se ha podido guardar la partida" << endl;
	}
      //Muestra los creditos y sale del juego
      if ( ok == 0 )
	{
	  cout << endl;
	  system ( "clear" );
	  cout << "  Street Master's v. " << version << "\n";
	  cout << "-------------------------\n";
	  cout << "Proyecto fin de carrera (ITIG)\n";
	  cout << "Autor: Javier Aragn Zabalegui\n";
	  cout << "Tutor: Carlos Linares Lpez\n";
	  cout << "Universidad Carlos III de Madrid\n\n";
	}
      return 0;
    }
  return 0;
}

/**
 * Mtodo que cierra una partida.
 * Para ello cierra la partida eliminando toda su informacin y devuelve el prompt del juego 
 * al estado inicial.
 */
void cerrarPartida ( tablero &partida, string &prompt )
{
  partida.cerrar();
  prompt = nombreJuego + "> ";
  return;
}

/**
 * Este comando cerrar la partida actual. Se ofrece la posibilidad de guardar la partida antes de salir.
 */
int com_cerrar ( char **argumentos, int num_args, tablero &partida, string &prompt )
{  
  int ok;

  partida.guardarLog ();
  if ( num_args > 1 )
    {
      cerr << "No se esperan argumentos para cerrar" << endl;
      ok = -1;
    }
  else
    {  
      //Ofrece la oportunidad de guardar antes de cerrar
      cout << "Quieres guardar antes de cerrar? (s/N)";
      char opcion = cin.get ();
      ok = 0;
      if ( opcion == 's' || opcion == 'S' )
	{
	  string aux;
	  cout << "Indica el nombre con el que se guardar la partida: ";
	  cin.clear();
	  cin >> aux;      
	  //Guardado es una cte que esta en configuracion.h
	  string fichero = guardado + aux + ".sav";
	  ok = partida.guardar ( fichero );
	}
      if ( ok == -1 )
	cerr << "No se ha podido guardar la partida" << endl;

      //Cierra la partida actual
      if ( ok == 0 )
	{
	  cerrarPartida ( partida, prompt );
	  cout << "Partida cerrada" << endl;	  
	}
    }
  return ok;
}

/**
 * Ejecuta una tirada:
 * Mueve la ficha y ejecuta la accin correspondiente a la casilla de destino. 
 */
int com_tirar ( char **argumentos, int num_args, tablero &partida, string &prompt )
{
  int ok;
  
  //Para poder tirar, el jugador que ejecuta el comando (jugador actual)
  //tiene que ser el mismo jugador al que le corresponde el turno.
  if ( partida.get_jugadorActual () == partida.get_turno () )   
    if ( num_args > 1 )
      {
	cerr << "ERROR: No se esperan argumentos para el comando 'tirar'." << endl;
	ok = -1;
      }
    else
      {
	//Mover al jugador (cambia el prompt del juego)
	system("clear");
	prompt = nombreJuego + " " + partida.moverJugador () + "> ";	  
	ok = 0;
      }   
  else
    {
      cerr << "No es tu turno para lanzar el dado" << endl;
      ok = -1;
    }
  return ok;
}

/**
 * Le dar el control del juego al jugador que posea el turno
 */
int com_fin ( char **argumentos, int num_args, tablero &partida, string &prompt )
{
  //Otorga el control al jugador que tiene que lanzar
  partida.set_jugadorActual ( partida.get_turno() );
  prompt = nombreJuego + " (" + partida.obtenerNombreJugador ( partida.get_turno() ) + ")> ";     
  return 0;
}

/**
 * Mtodo que permite a un jugador pagar sus deudas manualmente. Para ello slo deber teclear
 * el comando 'pagar' y este mtodo restar de su capital la cantidad estipulada en el atributo
 * estado de la instancia jugador y lo sumar directamente al capital del jugador benefactor
 */
int com_pagar ( char **argumentos, int num_args, tablero &partida, int &numJugadores )
{
  bool paga = true;
  int ok = 0;
  
  //Si existen deudas, de algun jugador, se pagarn inmediatamente
  if ( partida.existenDeudas( partida.get_jugadorActual() ) == true )
    {
      paga = partida.pagarDeudas();
      ok = 0;
    }
  else
    {
      cout << "Es admirable tu generosidad, pero no tienes ninguna deuda que pagar" << endl;
      ok = 0;      
    }
  if ( paga = false )
    {
      cout << "Si no puedes pagar tus deudas debes declararte en bancarrota para que el juego pueda continuar. Tus propiedades serán adjudicadas al jugador acreedor de tus deudas." << endl;
    }
  return ok;
}

/**
 * El jugador se declara en bancarrota siendo eliminado de la partida. Sus posesiones pasarn a ser parte
 * de su acreedor o se llevar a cabo una subasta si son varios o la banca. El turno pasa a ser del siguiente
 * jugador.
 */
int com_bancarrota ( char **argumentos, int num_args, tablero &partida, string &prompt, int &numJugadores )
{
  numJugadores = partida.declararBancarrota ( partida.get_jugadorActual() );
  prompt = nombreJuego + " (" + partida.obtenerNombreJugador ( partida.get_turno() ) + ")> ";
  cout << "Quedan " << numJugadores << " jugadores en la partida." << endl;
  return 0;
}

/**
 * Permite que un jugador que no posee el turno ejecute una accin. Se introduce el jugador al que
 * se quiere cambiar el control de la consola.
 */
int com_cambiar ( char **argumentos, int num_args, tablero &partida, string &prompt )
{
  int ok = 0;
  if ( num_args == 1 )
    cerr << "Se debe especificar un jugador" << endl;
  
  //Si se introduce un solo argumento,se entiende que quiere hacer el cambio segn el turno del
  //jugador, por ello debe ser un nmero positivo.
  else if ( num_args == 2 )    
    //buscar por turno
    {
      char* error;
      int num = strtol ( argumentos[1], &error, 10 );
      if ( ( argumentos[1] != NULL ) && ( strcmp(error,"") == 0 ) && ( num >= 0) )
	prompt = nombreJuego + partida.cambiarJugadorActual ( num ) + "> ";
      else 
	{
	  cerr << "Se espera un valor numrico positivo para el comando 'cambiar <turno>'" << endl;
	  ok = -1;   
	}
    }
  //Se puede introducir el valor a cambiar o bien por el nombre o bien por el color
  //En ambos casos se buscar entre los jugadores y se realizar la operacin.
  else if (num_args >= 3 )
    //buscar
    {
      string valor = "";
      for ( int i = 2; i < num_args; i++ )
	{
	  if ( i > 2 ) valor += " ";
	  valor += argumentos[i];
	}      
      if ( strcmp ( argumentos[1], "-color") == 0 ||  strcmp ( argumentos[1], "-nombre") == 0 )
	  prompt = nombreJuego + partida.cambiarJugadorActual ( argumentos[1], valor ) + "> ";	     
      else
	cout << "Opcin " << argumentos[1] << " no vlido" << endl;  
    } 
  return ( ok );
  
}

/**
 * Permite hipotecar una propiedad. Si no se introduce ninguna opcin para el comando
 * se mostrarn todas las propiedades hipotecables. Si se introduce un numero de casilla,
 * se hipotecara dicha casilla.
 */
int com_hipotecar ( char **argumentos, int num_args, tablero &partida )
{
  //Sin argumentos, mostrar todas las casillas hipotecables
  if ( num_args == 1 )
    cout << partida.verHipotecables();
  else if ( num_args == 2 )
    partida.hipotecar ("numero",argumentos[1]);
  return 0;
}

/**
 * Permite deshipotecar una propiedad hipotecada
 */
int com_deshipotecar ( char **argumentos, int num_args, tablero &partida )
{
  //Sin argumentos, mostrar todas las casillas hipotecadas por el jugador que introduce el comando
  if ( num_args == 1 )
    cout << partida.verHipotecados();
  else if ( num_args == 2 )
    partida.deshipotecar ("numero",argumentos[1]);    
  return 0;
}

/**
 * Permite construir sobre una calle determinada ( se indicar con el nmero de casilla ). Si no se
 * indica ningun numero de casilla se mostrarn todas las casillas contruibles.
 */
int com_construir ( char **argumentos, int num_args, tablero &partida )
{
  //Sin argumentos, mostrar todas las casillas hipotecadas por el jugador que introduce el comando
  if ( num_args == 1 )
    cout << partida.verConstruibles();
  else if ( num_args == 2 )
    partida.construir ("numero",argumentos[1]);    
  return 0;
}

/**
 * Permite vender una casa / hotel. Si no se indica el numero de casilla de la que se quiere eliminar
 * el edificio, se mostrarn todas las calles construidas, con su numero de casilla.
 */
int com_vender ( char **argumentos, int num_args, tablero &partida )
{
  if ( num_args == 1 )
    cout << partida.verConstruidas();
  else if ( num_args == 2 )
    partida.vender ("numero",argumentos[1]);
  return 0;
}

/**
 * Permite negociar a dos jugadores. El iniciador es el que hace la oferta.
 * No espera argumentos.
 */
int com_negociar ( char **argumentos, int num_args, tablero &partida )
{ 
  if ( num_args == 1 )
    partida.negociar();
  else 
    cerr << "No se esperan parametros para 'negociar'" << endl;
  return 0;
}

/**
 * Mtodo que guarda una partida en un fichero, que posteriormente se podr cargar. Se puede
 * introducir el fichero destino mediante un argumento del comando o bien cuando se pregunte
 * durante la ejecucin del mtodo.
 */
int com_guardar ( char **argumentos, int num_args, tablero &partida )
{
  string fichero;
  int ok;
  //Si no se ha indicado ningn fichero destino, se preguntar por uno al jugador.
  //Se indicar que el fichero es el que ha indicado aqui.
  if ( num_args == 1 )
    {
      string aux;
      cout << "Indica el nombre con el que se guardar la partida: ";
      cin.clear();
      cin >> aux;      
      //Guardado es una cte que esta en configuracion.h
      fichero = guardado + aux + ".sav";
      ok = 0;
    }
  // Si se pasa el fichero como argumento, se guardar directamente.
  // Se indica que el fichero ser el que esta en el primer argumento del
  // array de chars (argumentos[1])
  else if ( num_args == 2 )
    {
      fichero = guardado;
      fichero += argumentos[1];
      fichero += ".sav";      
      ok = 0;
    }
  else 
    {
      cerr << "No se esperan argumento para 'guardar'" << argumentos[3] << endl;
      ok = -1;
    }

  //Guardar
  if ( ok == 0 )
    {
      ok = partida.guardar ( fichero );
      cout << "fichero = " << fichero << endl;
    }    
  return 0;
}

/**
 * Mtodo que permite cargar una partida ya existente de Street Master's.  Se puede
 * introducir el fichero de origen mediante un argumento del comando o bien cuando se pregunte
 * durante la ejecucin del mtodo.
 */
int com_cargar (  char **argumentos, int num_args, tablero &partida, string& prompt )
{
  string fichero;
  int ok;

  //Si no ha indicado ningun fichero, se pedir que lo indique.
  //fichero ser el nombre del fichero.
  if ( num_args == 1 )
    {
      string aux;
      cout << "Indica el nombre de la partida que quiere cargar ( no indique .sav): ";
      cin >> aux;
      //Guardado es una cte que esta en configuracion.h
      fichero = guardado + aux + ".sav";
      ok = 0;
    }
  //Si se ha pasado como argumento, se indica dicho fichero como origen de la informacion
  else if ( num_args == 2 )
    {      
      fichero = guardado;
      fichero += argumentos[1];
      fichero += ".sav";
      ok = 0;
    }
  else 
    {
      cerr << "No se esperan argumento para 'guardar'" << argumentos[3] << endl;
      ok = -1;
    }
  //Cargar
  if ( ok == 0 )
    {
      //Si hay una partida abierta, se cierra antes de cargar la nueva.
      if ( partida.get_partidaAbierta () )
	{
	  cout << "Cerrando partida actual" << endl;
	  cerrarPartida ( partida, prompt );
	}
      string idPrompt;
      cout << "fichero = " << fichero << endl;
      ok = partida.cargar ( fichero, idPrompt );
      prompt = nombreJuego + " " + idPrompt + "> ";
      system ( "clear " );
    }    
  return 0;
}

/**
 * Comando minicomo. Esta opcion muestra un manual de usuario al jugador. Si se ha definido
 * la variable de entorno "MONOPOLY_BROWSER" se abrir un tutorial HTML en el navegador 
 * indicado por el usuario en dicha variable. Si no, se abrir en modo texto.
 */
int com_minicomo ()
{   
  //Comprueba si esta definida la variable de entorno.
  if ( getenv ("MONOPOLY_BROWSER") == NULL )
    //Si no se ha definido, se abre la ayuda en modo texto
    paginar_fichero ( MinicomoTxt, 32 );        
  else
    {
      pid_t pid;
      pid = fork();
      //Si se ha definido, se crea un proceso hijo, que ejecute la ayuda en el navegador
      if ( pid == 0 )
	execlp (  getenv ("MONOPOLY_BROWSER"),  getenv ("MONOPOLY_BROWSER"), IndexMinicomo, NULL );	
    }  
  cin.clear(); //limpio las banderas de cin. Colocarlas en estado de bueno
  cin.ignore( 2000, '\n' ); // vacia el buffer de la entrada ya que contiene "basura"
  return 0;
}

/**
 * Mtodo que muestra el histproal de comandos
 */
void com_history ()
{
  register int i;        // General purpose variable
  register HIST_ENTRY **lista;
  
  // Take the history list
  lista = history_list();
  
  // In case there is a list, ...
  if ( lista ) 
    {
      
      // Show all the history commands, ...
      for (i=0; lista[i]; i++)
	cout << " " << i+history_base << ": " << lista[i]->line << endl;      
      cout << endl;
    }
}

/**
 * Comando que muestra la utilidad esperada de la accin indicada para el jugador actual.
 * La cantidad minima / maxima (segn casos) ser la indicada como ltimo argumento.
 */
void verUtilidadEsperada ( char **argumentos, int num_args, tablero &partida )
{
  if ( num_args == 3)
    {
      if ( strcmp ( argumentos[1], "hipotecar" ) == 0 )
	{
	  partida.utilidadEsperadaHipotecar ( atol ( argumentos[2] ) );
	}
      if ( strcmp ( argumentos[1], "vender" ) == 0 )
	{
	  partida.utilidadEsperadaVender ( atol ( argumentos[2] ) );
	}
      if ( strcmp ( argumentos[1], "deshipotecar" ) == 0 )
	{
	  partida.utilidadEsperadaDeshipotecar ( atol ( argumentos[2] ) );
	}
      if ( strcmp ( argumentos[1], "construir" ) == 0 )
	{
	  partida.utilidadEsperadaConstruir ( atol ( argumentos[2] ) );
	}
    }
  else
    cerr << "Se espera una cantidad como argumento" << endl;
  return;
}

/**
 * Metodo que muestra el log determinado del jugador que se indique en ultimo lugar
 */
void com_log ( char **argumentos, int num_args, tablero partida )
{
  if ( partida.get_partidaAbierta() )
    {
      if ( num_args == 3)
	{
	  //Mostrar el log del jugador indicado en argumentos[1] y el tipo definido indicado 
	  //en argumentos[2] (dinero, capital, propiedades, edificios)
	  partida.mostrarLog ( argumentos[1], argumentos[2] );
	}
      else
	cerr << "Faltan argumentos: Esperado \"log color_jugador tipo_log\" " << endl;
    }
  else
    cerr << "Tienes que abrir una partida";
  return;  
}


/********************UTILIDADES PARA COMANDOS******************/

/**
 *  Identificar cual es el comando que se ha introducido en la linea de 
 *  comando, aislando tan solo la primera palabra, que debe ser el comando 
 *  Se eliminarn tanto los espacios del inicio como los del final, para 
 *  que la coincidencia sea total
 */
int identificar_comando ( char *word )
{		
  // Se busca una coincidencia con alguno de los nombres de 
  // los comandos, devolviendo su identificador de nmero
  if ( stripwhite_st ( word ) == "" )
    return -3;
  for ( int i=0; i <= total_comandos; i++ )
    {		
      if ( strcmp( word, comandos[i].nombre ) == 0 )
	return ( comandos[i].id );
    }
  
  // Si no ha encontrado ninguna coincidencia devolver -1
  // Es posible que se trate de un comando de linux 
  return -1;
}

/**
 * Ejecuta un comando en funcin de un identificador. Esta funcin ser
 * tan slo un switch que indique que tipo de comando se debe ejecutar.
 * Devuelve 2 cuando se llega al final de una partida
 */
int ejecutar_comando ( int id, char **argumentos, int num_args, tablero &partida, string &prompt )
{
  int ok = 0;
  int numeroJugadores = 6;	//Nmero de jugadores que existen en la partida
  
  switch ( id )
    {
    case 0: //salir
      ok = com_salir ( num_args, partida );
      break;

    case 1: //nuevo
      // Para crear una partida se deben cerrar primero todas las existentes.
      if ( partida.get_partidaAbierta () == false )
	{
	  ok = com_nuevo ( num_args, partida, prompt );
	  if (ok == 0)
	    partida.set_partidaAbierta( true );
	}
      else	
	  cerr << "ERROR: Ya existe una partida abierta, ciérrela antes de crear una nueva" << endl;	
      break;
      
    case 2: //ayuda
      ok = com_ayuda ( argumentos, num_args );
      break;

    case 3: //lista
      ok = com_lista ( argumentos, num_args );
      if ( ok == -1 ) //si sale con error finalizo el proceso.
	exit ( -1 );
      break;

    case 4: //ver
      //Para ver alguna informacin sobre la partida, debe existir alguna partida abierta
      if ( partida.get_partidaAbierta () == true )
	{
	  system ( "clear" );	  
	  ok = com_ver ( argumentos, num_args, partida );
	}
      else 
	cerr << "ERROR: No existen jugadores cargados. Abra una partida para crearlos" << endl;
      break;
    
    case 5: //cerrar
      //Para cerrar una partida, debe existir alguna ya abierta
      if ( partida.get_partidaAbierta () == true )
	ok = com_cerrar ( argumentos, num_args, partida, prompt );
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      break;
    
    case 6: //tirar dado
      // Para tirar los dados, debe existir una partida abierta.
      // Se lanzarn los dados y se ejecutar la accin de las casillas automticamente.
      if ( partida.get_partidaAbierta () == true ) 
	{
	  ok = com_tirar ( argumentos, num_args, partida, prompt );
	  if ( partida.get_numJugadores() == 1 )
	    {
	      cout << "Slo queda un jugador en la partida" << endl;
	      ok = 2; //Marca el fin de la partida.
	      cout << "Pulse 'intro' para continuar";
	      getchar();
	    }
	}
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      break;
    
    case 7: //fin de turno
      // Para trabajar con turnos, debe existir una partida abierta
      if ( partida.get_partidaAbierta () == true ) 
	ok = com_fin ( argumentos, num_args, partida, prompt );
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      break;
    
    case 8: //declarar la bancarrota
      //Debe existir una partida abierta y el jugador en cuestion no puede ser un cliente IA
      if ( partida.get_partidaAbierta () == true ) 
	{
	  if ( !partida.jugadorClienteIA() )
	    ok = com_bancarrota ( argumentos, num_args, partida, prompt, numeroJugadores );
	  else
	    cerr << "ERROR: No se puede ejecutar este comando porque el turno pertenece a un jugador automtico." << endl;
	}
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      if ( numeroJugadores == 1 )
	{
	  ok = 2; //Marca el fin de la partida.
	  cout << "Pulse 'intro' para continuar";
	  getchar();
	}
      break;
    
    case 9: //pagar
      // Debe existir una partida abierta para ejecutar este comando y adems no debe pertenecer el turno
      // a un agente IA
      if ( partida.get_partidaAbierta () == true )
	{
	  if ( !partida.jugadorClienteIA() )
	    ok = com_pagar ( argumentos, num_args, partida, numeroJugadores );
	  else
	    cerr << "ERROR: No se puede ejecutar este comando porque el turno pertenece a un jugador automtico." << endl;
	}
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      break;

    case 10: //Cambiar el jugador que posee la consola
      // Debe existir una partida abierta
      if ( partida.get_partidaAbierta () == true ) 
	ok = com_cambiar ( argumentos, num_args, partida, prompt );
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      break;      

    case 11: //Hipotecar
      // PAra ejecutar este comando la partida debe estar abierta y el turno no puede pertenecer
      // a un agente de IA
      if ( partida.get_partidaAbierta () == true ) 
	{
	  if ( !partida.jugadorClienteIA() )
	    ok = com_hipotecar ( argumentos, num_args, partida );
	  else
	    cerr << "ERROR: No se puede ejecutar este comando porque el turno pertenece a un jugador automtico." << endl;
	}
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      break;
     
    case 12: //Deshipotecar
      // Para ejecutar este comando la partida debe estar abierta y el turno no puede pertenecer
      // a un agente de IA
      if ( partida.get_partidaAbierta () == true ) 
	{
	  if ( !partida.jugadorClienteIA() )
	    ok = com_deshipotecar ( argumentos, num_args, partida );
	  else
	    cerr << "ERROR: No se puede ejecutar este comando porque el turno pertenece a un jugador automtico." << endl;
	}
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      break;

    case 13: //construir
      // Para ejecutar este comando la partida debe estar abierta y el turno no puede pertenecer
      // a un agente de IA
      if ( partida.get_partidaAbierta () == true ) 
	if ( !partida.jugadorClienteIA() )
	  ok = com_construir ( argumentos, num_args, partida );
	else
	  cerr << "ERROR: No se puede ejecutar este comando porque el turno pertenece a un jugador automtico." << endl;
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      break;

    case 14: //vender
      // Para ejecutar este comando la partida debe estar abierta y el turno no puede pertenecer
      // a un agente de IA
      if ( partida.get_partidaAbierta () == true ) 
	if ( !partida.jugadorClienteIA() )
	  ok = com_vender ( argumentos, num_args, partida );
      	else
	  cerr << "ERROR: No se puede ejecutar este comando porque el turno pertenece a un jugador automtico." << endl;
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      break;

    case 15: //negociar
      // Para ejecutar este comando la partida debe estar abierta y no puede existir ningun agente IA 
      // en la partida, porque los requisitos no reflejaban esta opcin.
      if ( partida.get_partidaAbierta () == true )
	if ( !partida.existeClienteIA() )	  
	  ok = com_negociar ( argumentos, num_args, partida );
	else
	  cerr << "ERROR: Accion no disponible cuando participa un jugador manejado por el ordenador." << endl;
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;
      break;
      
    case 16: //guardar
      //Debe existir una partida abierta para poder guardar la partida.
      if ( partida.get_partidaAbierta () == true ) 
	ok = com_guardar ( argumentos, num_args, partida );
      else 
	cerr << "ERROR: No existe ningua partida abierta actualmente." << endl;     
      break;

    case 17: //cargar
      //Debe existir una partida abierta para poder cargar la partida.
      ok = com_cargar ( argumentos, num_args, partida, prompt );
      if ( ok == -1 )
	cerr << "Error al cargar la partida" << endl;
      break;
      
    case 18: //minicomo      
      ok = com_minicomo ();
      break;

    case 19: //history
      //Historial de comandos
      com_history ();
      break;

    case 20: //ver utilidad esperada
      verUtilidadEsperada ( argumentos, num_args, partida );
      break;

    case 21: //Logs de cada jugador
      com_log ( argumentos, num_args, partida);
      break;	

    case -1: //Comando de linux
      ok = com_linux ( argumentos );
      if ( ok == -1 ) //si sale con error finalizo el proceso.
	exit ( -1 );
      break;
      
    case -2: //Comandos prohibidos dentro de street_master
      cerr << "Comando prohibido para Street Master's" << endl;
      ok = -1;
      break;

    }  
  
  return ok;
}

/**
 * Cuando la partida concluye, se muestra la informacin del ganador y se cierra la partida.
 */
void partidaTerminada ( tablero &partida, string &prompt )
{
  system ( "clear" );
  cout << "LA PARTIDA HA TERMINADO. TENEMOS UN GANADOR." << endl;
  partida.ver_jugador ( 0 );
  partida.guardarLog ();
  cerrarPartida ( partida, prompt );
  return;
}

/**
 * PROGRAMA PRINCIPAL
 */

/**
 * Mensaje de bienvenida al juego
 */
void bienvenida()
{
  cout << "*************************************************************" << endl;
  cout << "                   Street Master's v. " << version << endl;
  cout << "*************************************************************" << endl << endl;
  cout << ">Para obtener ayuda teclee 'ayuda'" << endl;
}

// Nmero mximo de argumentos que permite el interprete de mandatos
static const int max_argumentos = 10;
/* Programa principal */
/* NOTA: mejorar el máximo de argumentos, que ahora son 5 */
int main () 
{
  int num_args = 0, id_comando = -1, ok = 0;
  char **argumentos;
  char *linea;  
  tablero partida;
  string prompt = nombreJuego + "> ";
  long numComando = 0;
  long vAutosave = 1;
  bool repetir = false;

  system ( "clear" );
  bienvenida ();	
  // Inicializa readline. Debe ejecutarse al inicio del programa
  initialize_readline ();
  
  while ( ( ok != 0 ) || ( id_comando != 0 ) )
    {
      linea =  rl_gets ( prompt );		
      //Si no se lee nada en una linea no se har nada.
      if ( strcmp ( linea, "" ) != 0 )
	{
	  argumentos = new char*[max_argumentos];
	  num_args = obtener_argumentos ( linea, argumentos, max_argumentos );	 
	  id_comando = identificar_comando ( argumentos[0] );	  	    
	  numComando++;
	  //Autoguardar
	  if ( ( numComando > 10 ) && ( partida.get_partidaAbierta() ) )
	    {
	      char idAlpha[120];
	      sprintf( idAlpha, "%d", vAutosave );
	      string nombre = "saves/autosave" + string (idAlpha);
	      partida.guardar ( nombre );
	      numComando = 0;
	      vAutosave++;
	    }
	  ok = ejecutar_comando ( id_comando, argumentos, num_args, partida, prompt );
	  
	  //Los clientes de IA ejecutarn sus propias acciones despus de una tirada.
	  //Con esto se evita que se lancen continuamente tras consultar la ayuda, 
	  //ver el tablero u otros.	  
	  if ( partida.get_partidaAbierta () && id_comando == 6 )
	    {
	      string promptAux;
	      cout << "Los jugadores estn analizando su jugada" << endl;
	      partida.razonarJugadores ( promptAux );
	      prompt = nombreJuego + " " + promptAux + "> ";
	    }	
	  if ( ok == 2 || partida.get_numJugadores () == 1 ) 
	    partidaTerminada ( partida, prompt );
	  liberar_memoria ( argumentos, num_args );
	}
    }
  return 0;
}
