/***********************************************************************************
 *  CLASE TABLERO
 *  Clase que representa un tablero de street master's que es la encargada de gestionar
 *  una partida completa, que relaciona las casillas con los jugadores y el resto de
 *  elementos de una partida del juego.
 *  Esta clase adems de servir para comunicar dichas clases ser la encargada de guardar
 *  la informacin propia de una partida.
 *
 *  Autor: Javier Aragn Zabalegui (PFC para la uc3m: street Master's)
 *  Contacto: 100039831@alumnos.uc3m.es
 *  version 1.0
 **********************************************************************************/


#ifndef __TABLERO__
#define __TABLERO__

#include "cadenas.h"
#include "fichero.h"

#include "configuracion.h"

#include "jugador.h"
#include "jugador_reglas.h"
#include "calle.h"
#include "casilla.h"
#include "estacion.h"
#include "impuesto.h"
#include "impuesto5.h"
#include "salida.h"
#include "servicio.h"
#include "carcel.h"
#include "encarceladora.h"
#include "casilla_tarjetas.h"
#include "mazo.h"
#include "banca.h"

#include <vector>
#include <map>
#include <string>
#include <iostream>
#include <fstream>
using namespace std;

class tablero {
  
 private:
  //Atributos
  bool _partidaAbierta;
  string _nombrePartida;
  string _ciudad;
  string _moneda;
  string _fichCasillas;
  string _fichSuerte;
  string _fichCajaComunidad;  
  long _dineroInicial;
  int _numJugadores;
  static int _numCasillas;
  static int _salida;		//Casilla de salida
  static int _turno;
  int _jugadorActual;		//indica el jugador que controla el juego
  static bool _subastas;

  //Relaciones
  /**
     CONTIENE es vector de punteros porque:
	-vector: Para tener acceso directo al nmero de casilla
	-punteros: Para tratar los diferentes tipos de casillas (por la herencia)
  */
  static vector <casilla*> contiene; 
  static vector <jugador*> juega;
  map < string, mazo* > _mazo;
  static banca* _banca;
  
  /**
   * MTODOS PRIVADOS
   */
  /** 
   * Mtodos que obtienen los diferentes atributos de cada tipo de casilla. Los atributos 
   * vendrn dados por una cadena de caracteres (de la clase string de C++), y sern 
   * aislados mediante los mtodos de la familia obtener_substring_delimitado incluidos 
   * en la clase cadenas.    
   * Todos estos mtodos seguirn el mismo proceso:
   *  	- Se indica el valor en el que comienzan a leerse los atributos (tipicamente 
   *	la longitud del identificador del tipo de casilla + 1)
   *	- Se leer el primer atributo iniciandose en el valor indicado en el punto 
   *	anterior y se devolver la posicin donde comenzar el siguiente atributo.
   *	-En la siguiente lectura el inicio ser el final del anterior. Por ello en todos 
   *	las llamadas se irn 
   *	intercambiando las variables pos1 y pos2.
   */
  
  /**
   * Lee del fichero del tablero los datos referentes a una calle. Cada uno de estos
   * campos pasaran a formar parte de una instancia de casilla-calle 
   */
  int casilla_tipo_calle ( string linea );

  /** 
   * Lee del fichero del tablero los datos referentes a una casilla de propiedad de un servicio.
   * Cada uno de estos campos pasarn a formar parte de una instancia de casilla-propiedad-servicio 
   */
  int casilla_tipo_servicio ( string linea );
  
  /** 
   * Lee del fichero del tablero los datos referentes a una estacion. Cada uno de estos
   * campos pasaran a formar parte de una instancia de casilla-estacion 
   */
  int casilla_tipo_estacion ( string linea );

  /**
   * Lee del fichero del tablero los datos referentes a una casilla de impuestos.
   * Cada uno de estos campos pasaran a formar parte de una instancia de casilla-impuesto 
   */
  int casilla_tipo_impuesto ( string linea );
  
  /** 
   * Lee del fichero del tablero los datos referentes a un impuesto especial (con %). 
   * Cada uno de estos campos pasaran a formar parte de una instancia de 
   * casilla-impuesto-impuesto5 
   */
  int casilla_tipo_impuesto_especial ( string linea );
  
  /** 
   * Lee del fichero del tablero los datos referentes a una casilla de salida. 
   * Cada uno de estos campos pasaran a formar parte de una instancia de 
   * casilla-salida 
   */
  int casilla_tipo_salida ( string linea );

  /** 
   * Lee del fichero del tablero los datos referentes a una casilla de tipo parking. 
   * Las casillas de parking libre de pago, no tienen caractersticas propias, por 
   * lo que sern instancias directas de la clase CASILLA.
   */
  int casilla_tipo_parking ( string linea );
  
  /** 
   * Lee del fichero del tablero los datos referentes a una casilla de tipo "a la carcel".
   * Creara una instancia de la clase encarceladora y se incluir en el tablero
   */
  int casilla_tipo_encarceladora ( string linea );

  /** 
   * Lee del fichero del tablero los datos referentes a una casilla de tipo carcel. 
   * Con los datos obtenidos se crea una casilla de tipo carcel que se insetar
   * en el tablero.
   */
  int casilla_tipo_carcel ( string linea );
  
  /** 
   * Lee del fichero del tablero los datos referentes a una casilla de tarjetas. 
   * Cada uno de estos campos pasaran a formar parte de una instancia de 
   * casilla-casilla_tarjetas 
   */
  int casilla_tipo_tarjetas ( string linea );
  
 public:
  
  // Constructor por defecto
  tablero ();
  
  // Constructor explícito
  tablero ( string nombrePartida, string ciudad, string moneda, string fichCasillas, 
	    string fichSuerte, string fichCajaComunidad, int numJugadores, int numCasillas,
	    bool partidaAbierta, int salida, int turno, int jugadorActual, long dineroInicial, 
	    bool subastas );
  
  // Copy constructor
  tablero (const tablero& right);
  
  /* OPERADORES */

  // Operador de asignación  
  tablero& operator= (const tablero& right);
  casilla *operator[](unsigned int i);
  
  // Operador de salida
  /**
   * OPERADOR <<
   * Se imprimirn por pantalla todos aquellos tableros que estn formados por menos de 42
   * casillas. El resto mostrarn un mensaje para indicar que no es posible mostrarse (por
   * motivos de espacio.
   * Secuencia:
   *	1. Comprobar dimensiones del tablero: 
   *		- <=42: Dibujo el tablero
   *		- >42: No se dibuja porque no entra en la pantalla
   *	2. Dimensiones del tablero (por filas):
   *		1 fila: -> De lado a lado
   *			1 casilla (10x6 caracteres)
   *			resto (9x6 caracteres)
   *		de la fila 2 a la n-1
   *			casillas de (17x3 caracteres): una acoplada a cada esquina
   *			Fila n-1 dibuja el largo de la ltima fila.
   *		Fila n
   *			Igual que la 1 fila 
   */
  friend ostream& operator<< ( ostream& os, tablero &tab );

  // Destructor
  ~tablero ();
  
  // Mtodos GET
  string get_nombrePartida () const;  
  string get_ciudad () const;
  string get_moneda () const;
  string get_fichCasillas () const;
  string get_fichSuerte () const;
  string get_fichCajaComunidad () const;
  long get_dineroInicial () const;
  int get_numJugadores () const;
  static int get_numCasillas ();
  bool get_partidaAbierta () const;
  static int get_salida ();
  static int get_turno ();
  int get_jugadorActual () const;
  static banca *get_banca ();
  static bool get_subastas ();
  

  // Mtodos SET
  void set_nombrePartida ( string nombre );
  void set_ciudad ( string ciudad );
  void set_moneda ( string moneda );
  void set_fichCasillas ( string nombreFichero );
  void set_fichSuerte ( string nombreFichero );
  void set_fichCajaComunidad ( string nombreFichero );
  void set_dineroInicial ( long dineroInicial );
  void set_numJugadores ( int numJugadores );
  static void set_numCasillas ( int numCasillas );
  void set_partidaAbierta ( bool abierta );
  static void set_salida ( int salida );
  static void set_turno ( int turno );
  void set_jugadorActual ( int jugadorActual );
  void set_mazo ( string tipo, mazo* mazoTarjetas );
  static void set_banca ( banca *b );
  static void set_subastas ( bool subastas);
  
  //otros mtodos
  
  /**
   * Mtodo que retorna el nombre del jugador que ocupa la posicion indicada en el argumento 
   * 'numeroJugador'
   */
  string tablero::obtenerNombreJugador ( int numeroJugador );

  /** 
   * Mtodo que lee del fichero de casillas todas las casillas y crear la instancia de casillas
   * correspondiente 
   *  1- Leer del fichero una linea (casilla)
   *  2- Identificar que tipo de casilla es
   *  3- Crear la instancia 
   */
  int cargar_casillas ();

  /**
   * Mtodo que guarda la informacin sobre una partida en el fichero indicado. Se guardarn los datos
   * generales de la partida y los de los jugadores y casillas que se incluyen en ella.
   */
  int guardar ( string fichero );
  
  /**
   * Carga la liste de propiedades en un jugador. La lista viene dada con el formato:
   *     numeroPropiedad, numeroPropiedad, numeroPropiedad...
   */
  void cargarPropiedades ( jugador *j, string lista );

  /**
   * Carga la liste de tarjetas en un jugador. La lista viene dada con el formato:
   *     numeroTarjeta (mazo), numeroTarjeta (mazo)...
   */
  void cargarTarjetas ( jugador *j, string lista );

  /**
   * Mtodo que carga una partida ya comenzada. Se cargan los datos generales de la partida, adems de los datos 
   * de los jugadores, las propiedades y las tarjetas. Se debe indicar el fichero del que se carga y se devolver
   * el prompt del sistema en el estado que se qued cuando se guard la partida
   */
  int cargar ( string fichero, string &prompt );
  
  /**
   * Mtodo que comprueba uno por uno cada uno de los jugadores para comprobar que no
   * tenga ninguna deuda con otro jugador o con la banca, mediante el atributo estado,
   * que indica el estado financiero actual (debe o haber pendiente de pago/cobro).
   */
  bool existenDeudas ();
  
  /**
   * Mtodo derivado del anterior que comprueba slo si existen deudas para un jugador
   * concreto, que se pasar como parmetro indicando su numero de turno en el juego.
   */
  bool existenDeudas ( int idJugador);

  /**
   * Mtodo que muestra una lista de morosos en la partida. Para ser moroso un jugador debe
   * tener su atributo _estado negativo. Devuelve una cadena de caracteres con la informacion
   */
  string verMorosos ();
  
  /**
   * Mtodo que salda las deudas de juego que hay en la partida. El proceso consta de dos
   * partes: pagar deudas y cobrar deudas.
   *
   *	1. Pagar: Todos los jugadores con estado deudor (<0) pagarn la cantidad correspondiente.
   *		  Si no dipone de dinero suficiente no pagar, dejndole tiempo para que 
   *		  obtenga el dinero lquido necesario.
   *	2. Cobrar: Los jugadores con estado acreedor (>0) cobrarn la cantidad indicada.
   *
   * Se puede dar el caso de que los deudores no puedan pagar sus deudas. En este caso, los
   * cobradores evidentemente no podrn cobrar.
   */
  bool pagarDeudas ();

  /**
   * Mtodo que distribuye los bienes de un jugador eliminado entre el resto de los jugadores.
   * Para ello se subastarn los bienes del jugador eliminado. Si el acreedor es un unico
   * jugador y no es la banca, se la atribuyen los bienes directamente.
   */
  void redistribuirBienes ( jugador * j );

  /**
   * Mtodo que subasta una serie de propiedades que viene dada con una lista entre los acreedores indicados.
   * Se iran asignando las propiedades al jugador ganador en cada puja.
   */
  void subastarLista ( vector <jugador*> acreedores, list <propiedad*> listaPropiedades, jugador *propietario );

  /**
   * Mtodo que subasta una propiedad entre todos los jugadores interesados. Al final del proceso
   * la propiedad ser asignada a la banca si nadie la quiere o al jugador que ms alto puje.
   * Las pujas seguirn un orden secuencial, y un jugador podr pujar, pasar o salir de la subasta
   * del objeto
   */
  static void subastarPropiedad ( propiedad *p, vector<jugador*> interesados );
  
  /**
   * Mtodo que elimina un jugador de la partida. Para ello redistribuye sus bienes
   * y lo borra del tablero. A continuacin se retorna el nmero de jugadores que
   * quedan en la partida. Eliminar el jugador que ocupe la posicin del vector indicada
   */
  int declararBancarrota ( int numeroJugador);

  /**
   * Mtodo que elimina un jugador de la partida. Para ello redistribuye sus bienes
   * y lo borra del tablero. A continuacin se retorna el nmero de jugadores que
   * quedan en la partida. Eliminar el jugador que se indica.
   */
  int declararBancarrota ( jugador *j );

  /**
   * Mtodo que crea un tablero a partir de unas dimensiones dadas.
   * El resultado ser el tablero completo en una cadena de caracteres.
   * Cada casilla tendr el formato que se indica en el fichero ejemplo_casilla.txt
   */
  string dibujarTablero ( int casillasArriba, int casillasAbajo, int casillasDcha, int casillasIzqda );

  /**
   * Aade un jugador al vector de jugadores.
   * Recibe como parmetro un jugador y lo inserta en la ltima posicion del vector
   */
  void anyadir_jugador ( jugador* nuevo );

  /**
   * Mtodo que elimina un jugador del vector de jugadores que participan en una partida.
   * El jugador eliminado ser el indicado en la variable player
   */
  void eliminarJugador ( jugador* player );

  /**
   * Aade un puntero a una casilla al vector de casillas (contiene) del tablero.
   * La insercin se realiza en funcin del numero de la casilla, de forma que el
   * tablero queda ordenado.
   * Finalmente, se incrementa el nmero de casillas. 
   */  
  int anyadir_casilla ( casilla *nuevo );

  /**
   * Comrpueba si el jugador esta introduciendo un nombre repetido
   */
  bool repetido ( string nombre );
  
  /**
   * Mtodo que crear el jugador con los datos conocidos y llamar al mtodo que lo inserta
   * en el vector de jugadores.
   */
  int incluir_jugador ( int id, string nombre, string color, int jugadorAuto );

  /**
   * Aade una instancia de la clase banca a la partida, con los hoteles y casas iniciales determinadas
   * en los argumentos numCasas y numHoteles
   */
  int incluirBanca ( int numCasas, int numHoteles );

  /**
   *  Mtodo que recorre la lista de jugadores del tablero buscando aquel
   *  que responde al patron de color pasado como parmetro. Devuelve la instancia 
   *  de jugador localizada.
   */
  jugador* buscar_jugador_color ( string color ) const;

  /**
   *  Mtodo que recorre la lista de jugadores del tablero buscando aquel
   *  que responde al patron de color pasado como parmetro. Devuelve la posicin 
   *  del jugador localizado en el vector
   *  de jugadores.
   */
  int buscar_jugador_color_int ( string color ) const;

  /**
   *  Mtodo que recorre la lista de jugadores del tablero buscando aquel
   *  que responde al patron de nombre pasado como parmetro. Devuelve la posicin del 
   *  jugador localizado en el vector de jugadores.
   */
  int buscar_jugador_nombre_int ( string nombre ) const;
  
  
  // Metodos que crean una casilla y la incluyen dentro del tablero 

  /**
   * Metodo que crea una calle y la incluye dentro del tablero.
   * Adems relacionar la calle creada con su monopolio:
   *	- Si existe el monopolio, se relacionarn ambos objetos.
   *	- Si no existe, se crear el monopolio con la informacin existente.
   */
  void incluir_calle ( int numero_casilla, string nombre, string nombreCorto, string color, 
		       long precio, long hipoteca, long alquiler0, long alquiler1, long alquiler2, 
		       long alquiler3, long alquiler4, long alquiler_h, int precio_casa, 
		       int precio_hotel );

  /**
   * Metodo que crea un servicio y la incluye dentro del tablero.
   * Adems relacionar el servicio creado con su monopolio:
   *	- Si existe el monopolio, se relacionarn ambos objetos.
   *	- Si no existe, se crear el monopolio con la informacin existente.
   */
  void incluir_servicio ( int numero_casilla, string nombre, string nombreCorto, long precio, 
			  long hipoteca, int multiplicador1, int multiplicador2 );

  /**
   * Metodo que crea una estacin y la incluye dentro del tablero.
   * Adems relacionar la estacin creada con su monopolio:
   *	- Si existe el monopolio, se relacionarn ambos objetos.
   *	- Si no existe, se crear el monopolio con la informacin existente.
   */
  void incluir_estacion ( int numero_casilla, string nombre, string nombreCorto, long precio, 
			  long hipoteca, long alquiler1, long alquiler2, long alquiler3, 
			  long alquiler4 );

  /**
   * Mtodo que crea una casilla de impuestos y la incluye dentro del tablero 
   */
  void incluir_impuesto ( int numero_casilla, string nombre, string nombreCorto, int precio );

  /** 
   * Mtodo que crea una casilla de impuestos especiales y la incluye dentro del tablero  
   */
  void incluir_impuesto_especial ( int numero_casilla, string nombre, string nombreCorto, int precio, 
				   int tpc );

  /**
   * Mtodo que crea una casilla de tipo tarjetas y la incluye en en su posicin del tablero
   * Adems asociar la casilla a uno de los mazos creados en el tablero.
   */
  void incluir_casilla_tarjetas ( int numero_casilla, string nombre, string nombreCorto, string mazo );

  /**
   * Metodo que crea una casilla de tipo salida y la incluye dentro del tablero.
   */
  void incluir_salida ( int numero_casilla, string nombre, string nombreCorto, long sueldo );

  /**
   * Metodo que crea una casilla genrica y la incluye dentro del tablero.
   */
  void incluir_casilla_generica ( int numero_casilla, string nombre, string nombreCorto );

  /**
   * Metodo que crea una casilla de tipo crcel y la incluye dentro del tablero.
   */
  void incluir_carcel ( int numeroCasilla, string nombre, string nombreCorto, int maxTurnos, long multa );

  /**
   * Metodo que crea una casilla del tipo 'vaya a la crcel' y la incluye dentro del tablero.
   */
  void incluir_encarceladora ( int numeroCasilla, string nombre, string nombreCorto );

  /**
   * Mtodo que cierra la partida actual liberando memoria y estableciendo la partida como cerrada.
   * Una vez ejecutado, no podr seguir jugndose una partida.
   */
  void cerrar ();

  /**
   * Mtodo que reorganiza el tablero para que coincida el nmero de casilla con su
   * posicin en el vector.
   */
  int recolocar ( void );

  /** 
   *  Mtodo que verifica que la insercin de casillas ha sido correcta, es decir,
   *  coinciden los numeros de casilla con su orden en el vector 
   *  -Si estn bien colocadas = true
   *  -Si estn mal colocadas = false
   */
  bool verificar ( void );

  /** 
   * Mtodo que coloca un jugador en una casilla en funcin de los parmetros que se pasan:
   * La variable numJugador indica su orden dentro del array de jugadores y 'posicion'es el 
   * numero de casilla en la que se debe colocar.   
   */
  void colocarJugador ( int numJugador, int numCasilla );

  /** 
   *  Mtodo que establece que jugador iniciara la partida. Para ello lanzarn los dados cada
   *  uno de los jugadores dados de alta en la misma, lanzando una vez cada uno, y se seleccionar
   *  como primer jugador aquel que lance el dado y obtenga un nmero ms elevado. En caso de 
   *  empate, volvern a lanzar aquellos jugadores que hayan empatado en la tirada anterior.
   */
  string elegirOrdenTirada ( void );

  /**
   * Mtodo que dibuja una representacin del movimiento desde la casilla origen hasta
   * la casilla destino. Devolver una cadena de caracteres con este dibujo.
   */
  static string dibujarMovimiento ( int casillaOrigen, int casillaDestino );

  /**
   * Subastar una propiedad indicada como argumento
   */
  static int subastar ( propiedad *p);

  /**
   * Ofrece la libertad a un jugador que est en la crcel. Si tiene tarjetas para salir se le
   * ofrecer salir usandolas. Tambin se ofrece salir pagando la multa. Si el jugador sale se
   * le desencarcelar inmediatamente.
   */
  int ofrecerLibertad ( jugador* j );

  /**
   * Se evalua una accion retornada de las casillas. Es posible que el jugador espere para
   * conseguir dinero (1), no haga nada ms (0) o se declare en bancarrota (-1) 
   */
  string evaluarAccion ( jugador* j, int resultadoAccion );

  /**
   * Mtodo que mueve un jugador en funcin del resultado obtenido por la tirada de los 
   * dados. Se debe tener en cuenta cuando se lanza una tirada doble ( tambin cuando se
   * enviar al jugador a la crcel ).
   */
  string moverJugador ();
  
  /**
   * Cada jugador razonar si invierte dinero en algo o no. Para ello seguira el proceso: intentar
   * deshipotecar, intentar construir y si no hay dinero, declarar bancarrota.
   */
  void razonarJugadores ( string &prompt );

  /**
   * Muestra la leyenda del tablero, indicando todos los atributos de cada casilla y explicando
   * el significado de cada simbolo (se encuentra en un fichero de texto)
   */
  string leyenda ();

  /**
   * Crea un mazo de tarjetas y carga todas sus tarjetas desde el fichero correspondiente
   * al tipo de tarjetas que se pasa como argumento. El fichero se recoge de la instancia
   * de clase tablero.
   */
  int crearMazo ( string tipo );

  /**
   * Mtodo que muestra la informacin resumida sobre todos los jugadores 
   * de la partida actual en forma de tabla.
   */
  int ver_lista_jugadores ( );
  
  /** 
   *  Mtodo que muestra toda la informacin sobre el jugador indicado
   *  El identificador principal de los jugadores es el color, luego
   *  se buscar los jugadores por este atributo 
   */
  int ver_jugador ( string color );

  /** 
   * Mtodo que muestra toda la informacin sobre el jugador indicado
   * Se buscar al jugador por su turno (posicion en el vector 'juega')  
   */  
  int ver_jugador ( int pos );

  /**
   * Cambia el poseedor del control del juego al jugador con el turno que se indica en el argumento.
   * Este ser el jugador sobre el que recaeran las acciones hasta que se vuelva a cambiar.
   */
  string cambiarJugadorActual ( int nuevoJugadorActual );

  /**
   * Cambia el poseedor del control del juego al jugador que se pasa como argumento. Se deben
   * indicar el campo por el que se realizar el cambio y el valor de dicho campo.
   */
  string cambiarJugadorActual ( string campo, string valor );

  /**
   * Mtodo que muestra todas las propiedades hipotecables del jugador actual
   */
  string verHipotecables ();
  
  /**
   * Mtodo que muestra todas las propiedades hipotecadas del jugador actual
   */
  string verHipotecados ();

  /**
   * Mtodo que muestra todas las propiedades edificables del jugador actual
   */
  string verConstruibles ();  

  /**
   * Mtodo que muestra todas las calles edificadas del jugador actual
   */
  string verConstruidas ();

  /**
   * Mtodo que hipoteca la propiedad que se indica en la variable valor.
   * En esta versin solo se puede buscar por el atributo "numero".
   * Al hipotecar la propiedad el jugador recibe los ingresos automticamente.
   */
  int hipotecar ( string atributo, string valor );
  
  /**
   * Mtodo que deshipoteca la propiedad que se indica en la variable valor.
   * En esta versin solo se puede buscar por el atributo "numero".
   * Al deshipotecar la propiedad el jugador paga automticamente y su propiedad vuelve
   * a estar disponible para cobrar, construir...
   */
  int deshipotecar ( string atributo, string valor );

  /**
   * Mtodo que construye un edificio la propiedad que se indica en la variable valor.
   * En esta versin solo se puede buscar por el atributo "numero".
   * Al construir la propiedad el jugador paga automticamente y se aade el edificio
   * a la propiedad.
   * La propiedad en este caso debe ser de tipo calle.
   */
  int construir ( string atributo, string valor );

  /**
   * Mtodo que vende un edificio la propiedad que se indica en la variable valor.
   * En esta versin solo se puede buscar por el atributo "numero".
   * Al vender la propiedad el jugador cobra automticamente y se elimina el edificio
   * de la propiedad.
   * La propiedad en este caso debe ser de tipo calle.
   */
  int vender ( string atributo, string valor );
  
  /**
   * Mtodo que busca la siguiente propiedad del tipo indicada a partir de la posicion en la 
   * que se encuentra el jugador pasado como argumento del mtodo.
   */
  static casilla* buscarSiguienteCasillaTipo ( jugador* j,string tipo );

  /**
   * Mtodo que busca la siguiente propiedad del tipo indicada a partir de la posicion en la 
   * que se encuentra el jugador pasado como argumento del mtodo. Devuelve su posicion en el
   * vector de casilla.
   */
  static int buscarSiguienteCasillaTipoInt ( jugador* j, string tipo );
  
  /**
   * Metodo que mueve a un jugador a la casilla de carcel sin pagar por pasar por la salida.
   * Establece al jugador como encarcelado.
   */
  static void encarcelar ( jugador* j );

  /**
   * Mtodo que coloca a un jugador dado en una casilla dada
   */
  static void colocarJugador ( jugador* j, int numCasilla );
  
  /**
   * Mtodo que desliga al jugador de la casilla en la que se encuentra y le coloca
   * en la casilla indicada por el numero de casilla final
   */
  static int moverA ( jugador* j, int numCasillaFinal );  
  
  /**
   * Mtodo que mueve un jugador de una casilla a otra
   * Devuelve el resultado de la accin ejecutada en funcin de la casilla destino
   * Si pasa por la casilla de salida, se paga el sueldo
   */
  static int mover( jugador* j, int totalDados, bool tiradaDoble, bool salario );

  /**
   * Mtodo que mueve el jugador a la casilla indicada en la variable numCasillaFinal
   * Ejecuta la accin de la casilla destino
   */
  static int moverAAccion ( jugador* j, int numCasillaFinal );

  /**
   * Mtodo por el que un jugador paga la cantidad establecida al resto de jugadores.
   * Si se trata de una cantidad negativa cobrar dicha cantidad de los demas.
   * Los jugadores que no puedan pagar sern declarados morosos
   */
  static int pagarATodos ( jugador *j, long cantidad );

  /**
   * Pagar el sueldo si pasa por la casilla de salida y si está definida dicha casilla.
   */
  static void pagarSalario ( jugador *j, int numCasillaFinal );

  /**
   * Mtodo que muestra la informacin de la banca
   */
  void verBanca ();

  /**
   * Mtodo que muestra la informacin de la casilla indicada por la variable num
   */
  void verCasilla ( int num );
    
  /**
   * Mtodo que inicia un proceso de negociacin entre dos jugadores.
   * Se debe indicar el jugador receptor de la oferta
   */
  void negociar ();

  /**
   * Metodo que realiza un recuento del numero de estaciones libres que hay en el tablero
   */
  static int estacionesLibres ();

  /**
   * Metodo que realiza un recuento del numero de servicios ocupados que hay en el tablero
   */
  static int serviciosOcupados ();

  /**
   * Devolver el numero total de propiedades, siendo propiedades las calles, servicios y estaciones
   */
  static int totalPropiedades ();
  
  /**
   * Mtodo que calcula y devuelve el tanto por ciento de las casillas que estn libres
   */
  int propiedadesLibres ();
  
  /**
   * Mtodo que devuelve las casas disponibles en la banca
   */
  static int obtenerCasasDisponibles();

  /**
   * Mtodo que devuelve los hoteles disponibles en la banca
   */
  static int obtenerHotelesDisponibles();
  
  /**
   * Metodo que comprueba si el jugador actual es un jugador de IA (jugador_regas)
   */
  bool jugadorClienteIA ();

  /**
   * Mtodo que comprueba si alguno de los jugadores es un jugador de IA
   */
  bool existeClienteIA ();

  /**
   * Mtodo que muestra la mejor opcion para hipotecar siguiendo el criterio de la utilidad esperada
   * tomando como limite de dinero la cantidad introducida. El jugador sobre el que se aplica es el
   * jugador actual del juego.
   */
  void utilidadEsperadaHipotecar ( long cantidad );

  /**
   * Mtodo que muestra la mejor opcion para vender edificios siguiendo el criterio de la utilidad esperada
   * tomando como limite de dinero la cantidad introducida. El jugador sobre el que se aplica es el
   * jugador actual del juego.
   */
  void utilidadEsperadaVender ( long cantidad );

  /**
   * Mtodo que muestra la mejor opcion para deshipotecar siguiendo el criterio de la utilidad esperada
   * tomando como limite de dinero la cantidad introducida
   */
  void utilidadEsperadaDeshipotecar ( long cantidad );

  /**
   * Mtodo que muestra la mejor opcion para edificar siguiendo el criterio de la utilidad esperada
   * tomando como limite de dinero la cantidad introducida. El jugador sobre el que se aplica es el
   * jugador actual del juego.
   */
  void utilidadEsperadaConstruir ( long cantidad );

  /**
   * Mtodo que muestra el log indicado en el argumento tipoLog para el color que posee la ficha del colo
   * que indica el argumento colorJugador.
   */
  void mostrarLog ( string colorJugador, string tipoLog );

  /**
   * Mtodo que guarda en fichero los logs de todos jugadores que participan en la partida
   */
  void guardarLog ();  

};

/* GET */

inline string tablero::get_nombrePartida () const
{
  return _nombrePartida;
}

inline string tablero::get_ciudad () const
{
  return _ciudad;
}

inline string tablero::get_moneda () const
{
  return _moneda;
}

inline string tablero::get_fichCasillas () const
{
  return _fichCasillas;
}

inline string tablero::get_fichSuerte () const
{
  return _fichSuerte;
}

inline string tablero::get_fichCajaComunidad () const
{
  return _fichCajaComunidad;
}

inline long tablero::get_dineroInicial () const
{
  return _dineroInicial;
}

inline int tablero::get_numJugadores () const
{
  return _numJugadores;
}

inline int tablero::get_numCasillas ()
{
  return _numCasillas;
}

inline bool tablero::get_partidaAbierta () const
{
  return _partidaAbierta;
}

inline int tablero::get_salida ()
{
  return tablero::_salida;
}

inline int tablero::get_turno ()
{
  return tablero::_turno;
}

inline int tablero::get_jugadorActual () const
{
  return _jugadorActual;
}

inline banca *tablero::get_banca ()
{
  return _banca;
}

inline bool tablero::get_subastas ()
{
  return _subastas;
}

/* SET */

inline void tablero::set_nombrePartida ( string nombre )
{
  _nombrePartida = nombre;
}

inline void tablero::set_ciudad ( string ciudad )
{
  _ciudad = ciudad;
}

inline void tablero::set_moneda ( string moneda )
{
  _moneda = moneda;
}

inline void tablero::set_fichCasillas ( string nombreFichero )
{
  _fichCasillas = nombreFichero;
}

inline void tablero::set_fichSuerte ( string nombreFichero )
{
  _fichSuerte = nombreFichero;
}

inline void tablero::set_fichCajaComunidad ( string nombreFichero )
{
  _fichCajaComunidad = nombreFichero;
}

inline void tablero::set_dineroInicial ( long dineroInicial )
{
  _dineroInicial = dineroInicial;
}

inline void tablero::set_numJugadores ( int numJugadores )
{
  _numJugadores = numJugadores;
}

inline void tablero::set_numCasillas ( int numCasillas )
{
  _numCasillas = numCasillas;
}

inline void tablero::set_partidaAbierta ( bool abierta )
{
  _partidaAbierta = abierta;
}

inline void tablero::set_salida ( int salida )
{
  tablero::_salida = salida;
}

inline void tablero::set_turno ( int turno )
{
  tablero::_turno = turno;
}

inline void tablero::set_jugadorActual ( int jugadorActual )
{
  _jugadorActual = jugadorActual;
}

inline void tablero::set_mazo ( string tipo, mazo* mazoTarjetas )
{
  _mazo[tipo] = mazoTarjetas;
}

inline void tablero::set_banca ( banca *b )
{
  _banca = b;
}

inline void tablero::set_subastas ( bool subastas )
{
  _subastas = subastas;
}

#endif
